//
//  lcd_draw.c	Part of Graphic LCD module I/O library, Primitive figure display interface
//  Basic logic
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//		Primitive figure display.
//		Device not independ codes.
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.15 v0.0  Copy and modify from "s95160_2.c"
//    2022.03.03 v0.0  Added TRANSPARENT color support in lcd_drawChar().
//

#include "lcd_draw.h"					// API
#include "lcd_font.h"					// Pattern data

//
// Inline function
//
#define ABS(a)		((a) < 0 ? -(a) : (a))

//
//	Init
//
__attribute__((weak))					// overridable
void lcd_drawInit(void) {
	// Real devices implementation
	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);
	return;
}

//
//	Rectangle <DUMMY>
//
__attribute__((weak))					// overridable
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	// Real devices implementation
	return;
}

//
//	Image
//
__attribute__((weak))					// overridable
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				// 16bit color / pixel left->right top->down scan
) {
	int	i;
	int	j;

	for(j = 0; j < h; h++) {
		for(i = 0; i < w; i++) {
			lcd_drawRect(x+i, y+j, 1, 1, *image++);
		}
	}
	return;
}


//
//	Line
//
__attribute__((weak))					// overridable
void lcd_drawLine(
	short			x1,					// Start point
	short			y1,					//
	short			x2,					// End point
	short			y2,					//
	short			s,					// Line size
	unsigned short	color				//
) {
	short			w;
	short			h;
	short			i;

	w = (short)x2 - (short)x1 + 1;
	h = (short)y2 - (short)y1 + 1;
	x1 -= (s / 2);						// ADD 2014.03.09
	y1 -= (s / 2);						// ADD 2014.03.09

	if (ABS(w) < ABS(h)) {				// 45..90deg
		if (h < 0) {
			for(i = 0; i > h; i--) {
				short	x,y;

				x = x1 + (i * w / h);
				y = y1 + i;
				lcd_drawRect(x, y, s, s, color);
			}
		} else {
			for(i = 0; i < h; i++) {
				short	x,y;

				x = x1 + (i * w / h);
				y = y1 + i;
				lcd_drawRect(x, y, s, s, color);
			}
		}
	} else {							// 0..45deg
		if (w < 0) {
			for(i = 0; i > w; i--) {
				short	x,y;

				x = x1 + i;
				y = y1 + (i * h / w);
				lcd_drawRect(x, y, s, s, color);
			}
		} else {
			for(i = 0; i < w; i++) {
				short	x,y;

				x = x1 + i;
				y = y1 + (i * h / w);
				lcd_drawRect(x, y, s, s, color);
			}
		}
	}

	return;
}


//
//	Text
//
__attribute__((weak))					// overridable
void lcd_drawChar(
	short			x,
	short			y,
	unsigned short	fcolor,
	unsigned short	bcolor,
	char			c)
{
	int	i;
	int	j;
	const unsigned char	*p;

	if (x < 0) {
		x = 0;
	}
	if (y < 0) {
		y = 0;
	}
	p = &lcd_font5x7[((c & 0x7f) - ' ')][0];

	for(i = 0; i < LCD_FONT_HEIGHT; i++) {
		int	b;

		if (i < 7) {
			b = (*p++) << (8 - LCD_FONT_WIDTH);
		} else {
			b = 0;
		}
		for(j = 0; j < LCD_FONT_WIDTH; j++) {
			if (b & 0x80) {
//				lcd_drawRect(x+j, y, 1, 1, fcolor);			// DEL 2022.03.03
				if (fcolor != LCD_COLOR_TRANSPARENT) {		// ADD 2022.03.03
					lcd_drawRect(x+j, y, 1, 1, fcolor);		// ADD 2022.03.03
				}											// ADD 2022.03.03
			} else {
//				lcd_drawRect(x+j, y, 1, 1, bcolor);			// DEL 2022.03.03
				if (bcolor != LCD_COLOR_TRANSPARENT) {		// ADD 2022.03.03
					lcd_drawRect(x+j, y, 1, 1, bcolor);		// ADD 2022.03.03
				}											// ADD 2022.03.03
			}
			b <<= 1;
		}

		y++;
	}
	return;
}

__attribute__((weak))					// overridable
void lcd_drawStr(
	short			sx,					// Upper left position
	short			sy,					//
	unsigned short	fcolor,				// Text color
	unsigned short	bcolor,				// Background color
	char			*str				// '\0' terminated text
) {
	short	x;
	short	y;
	char	c;

	x  = sx;
	y  = sy;

	while((c = *str++) != '\0') {

		c &= 0b01111111;

		// Visual char
		if (c >= ' ') {
			lcd_drawChar(x, y, fcolor, bcolor, c);
			x += LCD_FONT_WIDTH;

		// CLF
		} else if (c == '\n') {
			x = sx;
			y += LCD_FONT_HEIGHT;

		// CR
		} else if (c == '\r') {
			y += LCD_FONT_HEIGHT;

		}

		// Screen range out
		if (x >= (LCD_SCREEN_WIDTH - LCD_FONT_WIDTH + 1)) {
			x = sx;
			y += LCD_FONT_HEIGHT;
		}
		if (y >= (LCD_SCREEN_HEIGHT - LCD_FONT_HEIGHT + 1)) {
			y = 0;
		}
	}

	return;
}

//
//	Text with font
//
__attribute__((weak))					// overridable
void lcd_drawChar2(
	short			x,
	short			y,
	unsigned short	fcolor,
	unsigned short	bcolor,
	char			c,
	const struct lcd_font *font)
{
	int	i;
	int	j;
	unsigned int	*pattern;

	if (x < 0) {
		x = 0;
	}
	if (y < 0) {
		y = 0;
	}
	pattern = lcd_fontGetPattern(font, c);
	for(i = 0; i < font->height; i++) {
		int	b;

		b = 1 << (font->width-1);
		for(j = 0; j < font->width; j++) {
			if (b & *pattern) {
//				lcd_drawRect(x+j, y, 1, 1, fcolor);			// DEL 2022.03.03
				if (fcolor != LCD_COLOR_TRANSPARENT) {		// ADD 2022.03.03
					lcd_drawRect(x+j, y, 1, 1, fcolor);		// ADD 2022.03.03
				}											// ADD 2022.03.03
			} else {
//				lcd_drawRect(x+j, y, 1, 1, bcolor);			// DEL 2022.03.03
				if (bcolor != LCD_COLOR_TRANSPARENT) {		// ADD 2022.03.03
					lcd_drawRect(x+j, y, 1, 1, bcolor);		// ADD 2022.03.03
				}											// ADD 2022.03.03
			}
			b >>= 1;
		}

		y++;
		pattern++;
	}
	return;
}

__attribute__((weak))					// overridable
void lcd_drawStr2(
	short			sx,					// Upper left position
	short			sy,					//
	unsigned short	fcolor,				// Text color
	unsigned short	bcolor,				// Background color
	char			*str,				// '\0' terminated text
	const struct lcd_font *font
) {
	short	x;
	short	y;
	char	c;

	x  = sx;
	y  = sy;

	while((c = *str++) != '\0') {

		c &= 0b01111111;

		// Visual char
		if (c >= ' ') {
			lcd_drawChar2(x, y, fcolor, bcolor, c, font);
			x += font->width;

		// CLF
		} else if (c == '\n') {
			x = sx;
			y += font->height;

		// CR
		} else if (c == '\r') {
			y += font->height;

		}

		// Screen range out
		if (x >= (LCD_SCREEN_WIDTH - font->width + 1)) {
			x = sx;
			y += font->height;
		}
		if (y >= (LCD_SCREEN_HEIGHT - font->height + 1)) {
			y = 0;
		}
	}

	return;
}

#include <math.h>
//
//	Arc
//
__attribute__((weak))					// overridable
void lcd_drawArc(
	short			x,					// Center point
	short			y,					//
	short			r,					//
	float			start,				//
	float			end,				//
	short			s,					// Line size
	unsigned short	color				//
) {
	int		x1;
	int		y1;
	int		x2;
	int		y2;
	float	a;

#define ARC_STEP (M_PI / 16.0)
	start += M_PI;
	end   += M_PI;
	x1 = cosf(start) * r + x;
	y1 = sinf(start) * r + y;
	for(a = start + ARC_STEP; a < end; a += ARC_STEP) {
		x2 = cosf(a) * r + x;
		y2 = sinf(a) * r + y;
		lcd_drawLine(x1, y1, x2, y2, s, color);
		x1 = x2;
		y1 = y2;
	}
	x2 = cosf(end) * r + x;
	y2 = sinf(end) * r + y;
	lcd_drawLine(x1, y1, x2, y2, s, color);
	return;
}


//
//	Open <DUMMY>
//
#include <stdio.h>
__attribute__((weak))					// overridable
void lcd_drawOpen(void)	{
	// Real devices implementation
	printf("lcd_drawOpen: default routine\n");
	lcd_drawInit();
	return;
}

