//
//  lcd_draw_driver_ILI9163V.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for ILI9325
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: ILITEK ILI9163V
//          Screen size   : 132x162, 128x128, 120x160, 128x160, 130x130, 132x132
//          Color mode    : 65K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          ATM0177B3A
//          M014C9163SPI
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, char2
//          LCD_SCREEN_WIDTH     : 162,160,132,130 or 128
//          LCD_SCREEN_HEIGHT    : 162,160,132,130 or 128
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation (auto define)
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//  Reference:
//      ILI TECHNOLOGY CORP.
//      a-Si TFT LCD Single Chip Driver 132RGBx162 Resolution and 262K color
//          Datasheet Specification
//              Version: V100
//              Document No.: ILI9163V_DTS_V100_20130416
//              Date: 2013.04.16
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.04 v0.0  First cording
//

#include "lcd_draw.h"					// External API
#include "lcd_draw_port.h"				// LCM port access subroutines
#include "lcd_spec_ILI9163V.h"			// Controller command define

#if defined(LCD_DRIVER_ILI9163V) || defined(LCD_DRIVER_ILI9163)

//
//	Option config
//
#define ARG_CHECK						// rect,image

//
//	Init
//
void lcd_drawInit(void) {
	unsigned char	cm[1] = {0
		| LCD_DATA_VIPF_16bitPixel
		| LCD_DATA_IFPF_16bitPixel		// 65k color
	};
	unsigned char	am[1] = {0
#if defined(LCD_LANDSCAPE_MODE) && defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MV
#elif defined(LCD_LANDSCAPE_MODE)
		| LCD_DATA_MX
		| LCD_DATA_MV
#elif defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MX
#else
//		| LCD_DATA_MY
//		| LCD_DATA_MX
//		| LCD_DATA_MV
//		| LCD_DATA_ML
//		| LCD_DATA_BGR
//		| LCD_DATA_MH
#endif
#ifdef LCD_MODULE_M014C9163SPI
	    | LCD_DATA_BGR
#endif
	};
#if defined(LCD_MODULE_ATM0177B3A)
	unsigned char	fr1[2] = {6, 20};	// DIVA,VPA   Frame_rate=200kHz/((Line+VPA)*(DIVA+4))
	unsigned char	gm[1]  = {2};		// GC 1,2,4,8  default=1
#else									//
	unsigned char	fr1[2] = {0, 0};	//
	unsigned char	gm[1]  = {0};		//
#endif

	lcd_portWriteCmd8(LCD_COMMAND_SoftwareReset);
	lcd_portWait(120);

	// Optional setting
	if (fr1[0] != 0) {
		lcd_portWriteCmdDatas8(LCD_COMMAND_FrameRateControl1, sizeof(fr1), fr1);
		lcd_portWriteCmdDatas8(LCD_COMMAND_GammaSet, sizeof(gm), gm);
	}

	// Required setting
	lcd_portWriteCmdDatas8(LCD_COMMAND_InterfacePixelFormat, sizeof(cm), cm);
	lcd_portWriteCmdDatas8(LCD_COMMAND_MemoryAccessControl, sizeof(am), am);
	lcd_portWriteCmd8(LCD_COMMAND_SleepOut);
	lcd_portWriteCmd8(LCD_COMMAND_DisplayOn);

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);

	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	int	i;
	short			e;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	e = x + w - 1;
	a[0] = x >> 8;
	a[1] = x;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_ColumnAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	e = y + h - 1;
	a[0] = y >> 8;
	a[1] = y;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_PageAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	lcd_portWriteCmd8(LCD_COMMAND_MemoryWrite);
	for(i = h * w; i > 0; --i) {
		lcd_portWriteData(color);
	}
	return;
}

//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				//
) {
	int	i;
	short			e;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	e = x + w - 1;
	a[0] = x >> 8;
	a[1] = x;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_ColumnAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	e = y + h - 1;
	a[0] = y >> 8;
	a[1] = y;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_PageAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	lcd_portWriteCmd8(LCD_COMMAND_MemoryWrite);
	for(i = h * w; i > 0; --i) {
		lcd_portWriteData(*image++);
	}
	return;
}

#endif	// LCD_DRIVER_ILI9163
