//
//  lcd_draw_driver_ILI9325.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for ILI9325
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: ILITEK ILI9325
//          Screen size   : 240 x 320
//          Color mode    : 262K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          CP2401T
//          S95417AAA
//          YHY024006A		-> Not verify
//          LCM240YP04		-> Not verify
//          QJ240084S0
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, Char2
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//          LCD_CHIP_ID_CHECK    : ID code(R00h) == 9325h
//          LCD_FORCE_8BIT_BUS   : 8bit data transfer by force
//
//  Refernce:
//      ILI TECHNOLOGY CORP.
//      a-Si TFT LCD Single Chip Driver 240RGBx320 Resolution and 262K color
//          Data sheet
//              Version: V0.43
//              Document No.: ILI9325DS_V0.43.pdf
//              Date: 2008.9.10
//          Application Notes
//              Version: Preliminary V0.22
//              Date: 2008.3.4
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.19 v0.0  First cording
//

#include "lcd_draw.h"					// External API
#include "lcd_draw_port.h"				// LCM port access subroutines
#include "lcd_spec_ILI9325.h"			// Controller command define

#ifdef LCD_DRIVER_ILI9325

//
//	Option config
//
//#define LCD_FORCE_8BIT_BUS			// 8bit data transfer by force
#define ARG_CHECK						// rect,image

#if defined(LCD_MODULE_CP2401T)
#define VC	7		//
#define AP	1		//
#define DC0	2		//
#define DC1	2		//
#define BT	4		//
#define VRH	8		//
#define VDV	15		//
#define VCM	16		//

#elif defined(LCD_MODULE_S95417AAA)
#define VC	7		//
#define AP	1		//
#define DC0	2		//
#define DC1	2		//
#define BT	4		//
#define VRH	8		//
#define VDV	24		//
#define VCM	37		//

#elif defined(LCD_MODULE_QJ240083S0)
#define VC	7		//
#define AP	1		//
#define DC0	2		//
#define DC1	2		//
#define BT	4		//
#define VRH	8		//
#define VDV	24		//
#define VCM	37		//

#else				// General settings
#define VC	7		// 0..7   *1.0   Vci1 = Vci * ((VC == 7) ? (1.0) : (0.95 - VC * 0.05))
#define AP	1		// 1..7   AP==1 -> Gamma driver amplifiers = 1.00, Source driver amplifiers = 1.00
#define DC0	2		// 0..6   1/4    Frequency = Fosc / (2 ^ DC0)
#define DC1	2		// 0..6   1/16   Frequency = Fosc / (2 ^ (DC1 + 2))
#define BT	4		// 1..7          see data sheet
#define VRH	12		// 1..15  *1.8   VREG1OUT = Vci * (VRH * 0.05 + ((VRH < 8) ? 1.95 : 1.2))
#define VDV	26		// 0..31  *1.14  Vcom amplitude = VREG1OUT * (VDV * 0.02 + ((VDV < 16) ? 0.7 : 0.62))
#define VCM	37		// 0..63  *0.87  VCOMH = VREG1OUT * (VCM * 0.005 + 0.685)
#endif


//
//	Init
//
void lcd_drawInit(void) {
	// Reset delay
	lcd_portWait(50);

#ifdef LCD_FORCE_8BIT_BUS
	// 7.2.4. i80/8-bit System Interface
	lcd_portWriteCmd(0);				// Data Transfer Synchronization
	lcd_portWriteCmd(0);
#endif

	// 12.4. Power Supply Configuration <Power on sequence>

	// Power On Reset and Display OFF
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl1, LCD_DATA_D(0));	// DTE=0,D=0,GON=0
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl3, 0);					// PON=0

	// Registers setting before power supply startup
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl2, LCD_DATA_VC(VC));
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl3, LCD_DATA_VRH(VRH));	// PON=0
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl7, LCD_DATA_VCM(VCM));
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl4, LCD_DATA_VDV(VDV));
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl1, LCD_DATA_BT(0));

	// Stabilizing time
	lcd_portWait(50);

	// Registers setting for power supply startup
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl1, LCD_DATA_BT(BT) | LCD_DATA_AP(AP) | LCD_DATA_APE);
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl2, LCD_DATA_VC(VC) | LCD_DATA_DC1(DC1) | LCD_DATA_DC0(DC0));
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl3, LCD_DATA_VRH(VRH) | LCD_DATA_PON);

	// Step-up circuit stabilizing time
	lcd_portWait(80);

	// Set the other registers (depend LCM)
#ifdef LCD_UPSIDE_DOWN_MODE
	lcd_portWriteCmdData(LCD_COMMAND_DriverOutputControl2,               LCD_DATA_NL(320) | LCD_DATA_SCN(0));
#else
	lcd_portWriteCmdData(LCD_COMMAND_DriverOutputControl2, LCD_DATA_GS | LCD_DATA_NL(320) | LCD_DATA_SCN(0));	// GS=H-mirror
	lcd_portWriteCmdData(LCD_COMMAND_DriverOutputControl1, LCD_DATA_SS);										// SS=V-Mirror
#endif
#ifdef LCD_LANDSCAPE_MODE
	lcd_portWriteCmdData(LCD_COMMAND_EntryMode, LCD_DATA_BGR | LCD_DATA_ID(2) | LCD_DATA_AM);
#else
	lcd_portWriteCmdData(LCD_COMMAND_EntryMode, LCD_DATA_BGR | LCD_DATA_ID(3));
#endif
	lcd_portWriteCmdData(LCD_COMMAND_BaseImageDisplayControl, LCD_DATA_REV);

	// 12.2 Display ON/OFF Sequence <Display On Flow>
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl1, LCD_DATA_BT(BT) | LCD_DATA_AP(AP) | LCD_DATA_SAP | LCD_DATA_APE);
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl1, LCD_DATA_D(1));
	lcd_portWait(1);
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl1, LCD_DATA_D(1) | LCD_DATA_GON);
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl1, LCD_DATA_D(3) | LCD_DATA_GON);
	lcd_portWait(1);
//	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl1, LCD_DATA_D(3) | LCD_DATA_GON | LCD_DATA_DTE);		// Not enough
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl1, LCD_DATA_D(3) | LCD_DATA_GON | LCD_DATA_DTE | LCD_DATA_BASEE);

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);
	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

#ifdef LCD_LANDSCAPE_MODE
#define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_HorizentalGRAMAddressSet, LCD_SCREEN_HEIGHT-(y)-1);lcd_portWriteCmdData(LCD_COMMAND_VerticalGRAMAddressSet, (x))
#else
#define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_HorizentalGRAMAddressSet, (x)); lcd_portWriteCmdData(LCD_COMMAND_VerticalGRAMAddressSet, (y))
#endif

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	int	i;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK
	for(; h > 0; --h) {
		lcd_portSetAddr(x, y);
		lcd_portWriteCmd(LCD_COMMAND_WriteDataToGRAM);
		for(i = w; i > 0; --i) {
			lcd_portWriteData(color);
		}
		y++;
	}
	return;
}

//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height 0> is reverse(down->up)
	unsigned short	*image				// 16bit color / pixel left->right top->down scan
) {
	int	i;
	int	j;
	int	f;

	if (h < 0) {						// Flip mode
		h = -h;
		f = -1;
	} else {
		f = 1;
	}

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif //ARG_CHECK

	if (f < 0) {
		y += h;
		--y;
	}
	for(j = h; j > 0; --j) {
		lcd_portSetAddr(x, y);
		lcd_portWriteCmd(LCD_COMMAND_WriteDataToGRAM);
		for(i = w; i > 0; --i) {
			lcd_portWriteData(*image++);
		}
		y += f;
	}
	return;
}

#endif	// LCD_DRIVER_ILI9325
