//
//  lcd_draw_driver_ILI9341.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for ILI9325
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: ILITEK ILI9341
//          Screen size   : 240x320
//          Color mode    : 65K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          MSP2807
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, char2
//          LCD_SCREEN_WIDTH     : 240
//          LCD_SCREEN_HEIGHT    : 320
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation (auto define)
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//  Reference:
//      ILI TECHNOLOGY CORP.
//      a-Si TFT LCD Single Chip Driver 240RGBx320 Resolution and 262K color
//          Datasheet Specification
//              Version     : V1.09
//              Document No.: ILI9341_DS_V1.09.pdf
//              Date        : 2011.03.15
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.04 v0.0  First cording
//

#include "lcd_draw.h"					// External API
#include "lcd_draw_port.h"				// LCM port access subroutines
#include "lcd_spec_ILI9341.h"			// Controller command define

#if defined(LCD_DRIVER_ILI9341)

//
//	Option config
//
#define ARG_CHECK						// rect,image

//
//	Init
//
void lcd_drawInit(void) {
	unsigned char	cm[1] = {0
		| LCD_DATA_DPI_16bit
		| LCD_DATA_DBI_16bit			// 65k color
	};
	unsigned char	am[1] = {0
#if defined(LCD_LANDSCAPE_MODE) && defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MV
#elif defined(LCD_LANDSCAPE_MODE)
		| LCD_DATA_MX
		| LCD_DATA_MV
#elif defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MX
#endif
#ifdef LCD_MODULE_MSP2807
		| LCD_DATA_BGR
#endif
	};

	lcd_portWriteCmd8(LCD_COMMAND_SoftwareReset);
	lcd_portWait(120);

	// Optional setting
#ifdef LCD_MODULE_MSP2807
	{
		unsigned char	df[4] = {
				LCD_DATA1_PTG_IntervalScan | LCD_DATA1_PT_AGND_AGND_AGND_AGND,
				LCD_DATA2_SS_S720_S1 | LCD_DATA2_REV_NormallyWhite | LCD_DATA2_ISC(2),
				LCD_DATA3_NL(39),
				LCD_DATA4_PCDIV(0)
		};

		lcd_portWriteCmdDatas8(LCD_COMMAND_DisplayFuctionControl, sizeof(df), df);
	}
#endif

	// Required setting
	lcd_portWriteCmdDatas8(LCD_COMMAND_PixelFormatSet, sizeof(cm), cm);
	lcd_portWriteCmdDatas8(LCD_COMMAND_MemoryAccessControl, sizeof(am), am);
	lcd_portWriteCmd8(LCD_COMMAND_SleepOut);
	lcd_portWriteCmd8(LCD_COMMAND_DisplayON);

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);

	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	int	i;
	short			e;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	e = x + w - 1;
	a[0] = x >> 8;
	a[1] = x;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_ColumnAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	e = y + h - 1;
	a[0] = y >> 8;
	a[1] = y;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_PageAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	lcd_portWriteCmd8(LCD_COMMAND_MemoryWrite);
	for(i = h * w; i > 0; --i) {
		lcd_portWriteData(color);
	}
	return;
}

//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				//
) {
	int	i;
	short			e;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	e = x + w - 1;
	a[0] = x >> 8;
	a[1] = x;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_ColumnAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	e = y + h - 1;
	a[0] = y >> 8;
	a[1] = y;
	a[2] = e >> 8;
	a[3] = e;
	lcd_portWriteCmd8(LCD_COMMAND_PageAddressSet);
	lcd_portWriteDatas8(sizeof(a), a);

	lcd_portWriteCmd8(LCD_COMMAND_MemoryWrite);
	for(i = h * w; i > 0; --i) {
		lcd_portWriteData(*image++);
	}
	return;
}

#endif	// LCD_DRIVER_ILI9341
