//
//  lcd_draw_driver_S1D15G10.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for S1D15G10
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: Seiko Epson S1D15G10
//          Screen size   : 132 x 132
//          Color mode    : 4096
//
//      Support LCD Module (LCM):
//          L1F10085
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, Char2
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//  Refernce:
//      Seiko Epson Corporation
//      S1D15G10D08B000
//      132 RGB x 132 CSTN color LCD driver
//          Data sheet
//              Version     : Rev. 1.0
//              Document No.: MF1493-03
//              Date        : 2002
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.06 v0.0  First cording
//

#include "lcd_draw.h"					// API
#include "lcd_draw_port.h"				// Port I/O
#include "lcd_spec_S1D15G10.h"			// Controller command parameter

#ifdef LCD_DRIVER_S1D15G10

//
//	Init
//
void lcd_drawInit(void) {
	unsigned char	c3[] = {0b00000001};	// 0b001: COM1->68 132->69pin
	unsigned char	c4[] = {0b00001111};	// LCD drive power switch
	unsigned char	c5[] = {0b00000010,		// Address scan direction
			                0b00000001,		// RGB arrangement
							0b0000100};		// color mode: 0b001=256 0b010=4096(typeA) 0b100=4096(TypeB)
	unsigned char	c7[] = {40,				// V1 volume value alpha
			                3};				// 1 + Rb/Ra

#ifdef LCD_LANDSCAPE_MODE
  #ifdef LCD_UPSIDE_DOWN_MODE
	c5[0] = 0b00000100;
  #else
	c5[0] = 0b00000111;
  #endif
#else
  #ifdef LCD_UPSIDE_DOWN_MODE
	c5[0] = 0b00000001;
  #endif
#endif

	lcd_portWait(100);
	lcd_portWriteCmdDatas8(LCD_CMD_COMSCN, sizeof(c3), c3);
	lcd_portWriteCmd8     (LCD_CMD_OSCON);
	lcd_portWriteCmd8     (LCD_CMD_SLPOUT);
	lcd_portWriteCmdDatas8(LCD_CMD_PWRCTR, sizeof(c4), c4);
//	lcd_portWriteCmd8     (LCD_CMD_DISNOR);
	lcd_portWriteCmd8     (LCD_CMD_DISINV);
	lcd_portWriteCmdDatas8(LCD_CMD_DATCTL, sizeof(c5), c5);
	lcd_portWriteCmdDatas8(LCD_CMD_VOLCTR, sizeof(c7), c7);
	lcd_portWait(100);
	lcd_portWriteCmd8     (LCD_CMD_DISON);

	// Screen clear
	lcd_drawRect(0, 0, LCD_SCREEN_WIDTH, LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);

	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

//
//	Local
//
void lcd_drawSetWindow(short x, short y, short w, short h){
	unsigned char	a[2];
	unsigned char	b[2];
	short			xe;
	short			ye;

	xe = x + w - 1;
	ye = y + h - 1;
	#ifdef LCD_LANDSCAPE_MODE
	  #ifdef LCD_UPSIDE_DOWN_MODE
		a[0] = y  + 2;
		a[1] = ye + 2;
		b[0] = x  + 2;
		b[1] = xe + 2;
	  #else
		a[0] = y;
		a[1] = ye;
		b[0] = x;
		b[1] = xe;
	  #endif
	#else
	  #ifdef LCD_UPSIDE_DOWN_MODE
		a[0] = x  + 2;
		a[1] = xe + 2;
		b[0] = y;
		b[1] = ye;
	  #else
		a[0] = x;
		a[1] = xe;
		b[0] = y  + 2;
		b[1] = ye + 2;
	  #endif
	#endif
	lcd_portWriteCmdDatas8(LCD_CMD_CASET, sizeof(a), a);
	lcd_portWriteCmdDatas8(LCD_CMD_PASET, sizeof(b), b);
	return;
}
unsigned short lcd_drawColorConv(unsigned short color)	{
	unsigned short	c;

	c = ((color & 0b1111000000000000) >> 4)
	  | ((color & 0b0000011110000000) >> 3)
	  | ((color & 0b0000000000011110) >> 1);
	return(c);
}

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	int				i;
	unsigned short	c;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	lcd_drawSetWindow(x, y, w, h);

	c = lcd_drawColorConv(color);
	lcd_portWriteCmd8(LCD_CMD_RAMWR);
	for(i = w * h; i > 0; --i) {
		lcd_portWriteData(c);
	}

	return;
}

//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				//
) {
	short	i;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	lcd_drawSetWindow(x, y, w, h);

	lcd_portWriteCmd8(LCD_CMD_RAMWR);
	for(i = w * h; i > 0; --i) {
		lcd_portWriteData(lcd_drawColorConv(*image++));
	}

	return;
}

#endif
