//
//  lcd_draw_driver_SSD1283A.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for SSD1283A
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: Solomon Systech SSD1283A
//          Screen size   : 132 x 132
//          Color mode    : 65K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          H161T01
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, Char2
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//  Refernce:
//      SOLOMON SYSTECH
//      SSD1283A
//      132 RGB x 132 TFT Driver
//      Integrated Power, Gate, Source Driver with built-in RAM
//          Data sheet
//              Version     : Rev 0.12
//              Document No.: none
//              Date        : Jul 2005
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.06 v0.0  First cording
//

#include "lcd_draw.h"					// API
#include "lcd_draw_port.h"				// Port I/O
#include "lcd_spec_SSD1283A.h"			// Controller command parameter

#ifdef LCD_DRIVER_SSD1283A

//
//	Option config
//
#define ARG_CHECK				// rect,image

//
// Inline function
//
#ifdef LCD_MODULE_H161T01
#define OFFS	2
#else
#define OFFS	0
#endif

#ifdef LCD_LANDSCAPE_MODE
  #ifdef LCD_UPSIDE_DOWN_MODE
    #define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSet, LCD_DATA_X((y)+OFFS) | LCD_DATA_Y(LCD_SCREEN_WIDTH-(x)-1+OFFS))
  #else
    #define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSet, LCD_DATA_X(LCD_SCREEN_HEIGHT-(y)-1+OFFS) | LCD_DATA_Y((x)+OFFS))
  #endif
#else
  #ifdef LCD_UPSIDE_DOWN_MODE
    #define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSet, LCD_DATA_X(LCD_SCREEN_WIDTH-(x)-1+OFFS) | LCD_DATA_Y(LCD_SCREEN_HEIGHT-(y)-1+OFFS))
  #else
    #define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSet, LCD_DATA_X((x)+OFFS) | LCD_DATA_Y((y)+OFFS))
  #endif
#endif

//
// Init
//
void lcd_drawInit(void) {
	unsigned int	a;

	lcd_portWriteCmdData(LCD_COMMAND_PowerControl1, LCD_DATA_PowerControl1
//			| LCD_DATA_DOT
			| LCD_DATA_DCY_FlineX2
			| LCD_DATA_BTH_unregulated
			| LCD_DATA_DC_fosc
			| LCD_DATA_AP_SmallToMedium
//			| LCD_DATA_SLP
			);

	lcd_portWriteCmdData(LCD_COMMAND_PowerControl2, LCD_DATA_PowerControl2
//			| LCD_DATA_PU_X3
			| LCD_DATA_PU_X4
//			| LCD_DATA_PU_X5
//			| LCD_DATA_PU_X6
			);

	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl, LCD_DATA_DisplayControl
//			| LCD_DATA_PT(0)
//			| LCD_DATA_VLE1
//			| LCD_DATA_VLE2
//			| LCD_DATA_SPT
			| LCD_DATA_GON
			| LCD_DATA_DTE
//			| LCD_DATA_CM
			| LCD_DATA_D1
			| LCD_DATA_D0
			);

	lcd_portWriteCmdData(LCD_COMMAND_Oscillator, LCD_DATA_OSCEN);
//	lcd_portWait(100);

	lcd_portWriteCmdData(LCD_COMMAND_DriverOutputControl, LCD_DATA_DriverOutputControl
			| LCD_DATA_REV
//			| LCD_DATA_CAD
//			| LCD_DATA_BGR
//			| LCD_DATA_SM
//			| LCD_DATA_TB				// Zero point is up & left
			| LCD_DATA_RL				// S6 to S395
			| LCD_DATA_MUX(131)
			);

	a = LCD_DATA_EntryMode
//			| LCD_DATA_VSMode
			| LCD_DATA_DFM_65k
//			| LCD_DATA_DFM_262k
//			| LCD_DATA_TRANS
			| LCD_DATA_OEDef
//			| LCD_DATA_WMode
			| LCD_DATA_DMode_Ram
//			| LCD_DATA_DMode_GenericInput
//			| LCD_DATA_TY_TypeA
//			| LCD_DATA_TY_TypeB
//			| LCD_DATA_TY_TypeC
//			| LCD_DATA_ID_HdecVdec
//			| LCD_DATA_ID_HincVdec
//			| LCD_DATA_ID_HdecVinc
//			| LCD_DATA_ID_HincVinc
//			| LCD_DATA_AM_H
//			| LCD_DATA_AM_V
			| LCD_DATA_LG(0)
			;
#ifdef LCD_LANDSCAPE_MODE
	a |= LCD_DATA_AM_V;
#else
	a |= LCD_DATA_AM_H;
#endif
#ifdef LCD_UPSIDE_DOWN_MODE
	a |= LCD_DATA_ID_HdecVdec;
#else
	a |= LCD_DATA_ID_HincVinc;
#endif
	lcd_portWriteCmdData(LCD_COMMAND_EntryMode, a);

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);

	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	short	i;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	for(; h > 0; --h) {
		lcd_portSetAddr(x, y);
		lcd_portWriteCmd(LCD_COMMAND_WriteDataToGRAM);
		for(i = w; i > 0; --i) {
			lcd_portWriteData(color);
		}
		y++;
	}
	return;
}


//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				// 16bit color / pixel left->right top->down scan
) {
	short	i;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	for(; h > 0; --h) {
		lcd_portSetAddr(x, y);
		lcd_portWriteCmd(LCD_COMMAND_WriteDataToGRAM);
		for(i = w; i > 0; --i) {
			lcd_portWriteData(*image++);
		}
		y++;
	}
	return;
}

#endif	// LCD_DRIVER_SSD1283A
