//
//  lcd_draw_driver_SSD1289.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for SSD1289
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: Solomon Systech SSD1289
//          Screen size   : 240 x 320
//          Color mode    : 262K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          S95160
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, Char2
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//  Refernce:
//      SOLOMON SYSTECH
//      SSD1289
//      240 RGB x 320 TFT LCD Controller Driver
//      integrated Power Circuit, Gate and Source Driver
//          Data sheet
//              Version     : Rev 1.3
//              Document No.: none
//              Date        : 2007
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.03 v0.0  First cording
//

#include "lcd_draw.h"					// API
#include "lcd_draw_port.h"				// Port I/O
#include "lcd_spec_SSD1289.h"			// Controller command parameter

#ifdef LCD_DRIVER_SSD1289

//
//	Option config
//
#define ARG_CHECK				// rect,image

//
// Init
//
void lcd_drawInit(void) {
#ifdef LCD_UPSIDE_DOWN_MODE
	lcd_portWriteCmdData(LCD_COMMAND_DriverOutputControl, 0x793f);
#else
	lcd_portWriteCmdData(LCD_COMMAND_DriverOutputControl, 0x3b3f);
#endif
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl,      0x0021);	// GON=1,DTE=0,D=1
	lcd_portWriteCmdData(LCD_COMMAND_Oscillator,          0x0001);	// OSCEN=1
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl,      0x0023);	// GON=1,DTE=0,D=3
	lcd_portWriteCmdData(LCD_COMMAND_SleepMode,           0x0000);	// Exit sleep

	lcd_portWait(30);
	lcd_portWriteCmdData(LCD_COMMAND_DisplayControl,      0x0033);	// GON=1,DTE=1,D=3
#ifdef LCD_LANDSCAPE_MODE
	lcd_portWriteCmdData(LCD_COMMAND_EntryMode,           0x6838);
#else
	lcd_portWriteCmdData(LCD_COMMAND_EntryMode,           0x6830);
#endif

#ifdef LCD_MODULE_S95160
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl1,       0x6664);		// 6664 DCT,BT,DC,AP
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl2,       0x0004);		// 0004 VRC=0..7
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl3,       0x0006);		// 0009 VRH=0..15
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl4,       0x3000);		// 3200 VCOMG,VDV
	lcd_portWriteCmdData(LCD_COMMAND_PowerControl5,       0x0029);		// 0029 VCM=0..63
	lcd_portWriteCmdData(LCD_COMMAND_FrameFrequencyControl, 0xe000);	// 8000 OSC=0,2,5,8,10,12,14
#endif

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);

	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}


#ifdef LCD_LANDSCAPE_MODE
#define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSetX, LCD_SCREEN_HEIGHT-(y)-1);lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSetY, (x))
#else
#define lcd_portSetAddr(x,y)	lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSetX, (x));lcd_portWriteCmdData(LCD_COMMAND_RAMaddressSetY, (y))
#endif

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	short	i;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	for(; h > 0; --h) {
		lcd_portSetAddr(x, y);
		lcd_portWriteCmd(LCD_COMMAND_WriteDataToGRAM);
		for(i = w; i > 0; --i) {
			lcd_portWriteData(color);
		}
		y++;
	}
	return;
}


//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				// 16bit color / pixel left->right top->down scan
) {
	short	i;

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif // ARG_CHECK

	for(; h > 0; --h) {
		lcd_portSetAddr(x, y);
		lcd_portWriteCmd(LCD_COMMAND_WriteDataToGRAM);
		for(i = w; i > 0; --i) {
			lcd_portWriteData(*image++);
		}
		y++;
	}
	return;
}

#endif	// LCD_DRIVER_SSD1289
