//
//  lcd_draw_driver_ST7735S.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for ST7735S
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: Sitronix ST7735S
//          Screen size   : 132x162
//          Color mode    : 65K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          Z144SN005
//          Z180SN009
//          Z180SN1720
//          ZY-FDG1442701V	<- Not verify
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, char2
//          LCD_SCREEN_WIDTH     : 162,160, 132 or 128
//          LCD_SCREEN_HEIGHT    : 162,160, 132 or 128
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//   Reference:
//      Sitronix Technology Corp.
//      132RGB x 162dot 262K Color with Frame Memory
//      Single-Chip TFT Controller/Driver
//          Datasheet
//              Version     : 1.1
//              Document No.: none
//              Date        : 2011/11
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.07 v0.0  First cording
//


#include "lcd_draw.h"					// API
#include "lcd_draw_port.h"				// Port I/O
#include "lcd_spec_ST7735S.h"			// Command & parameter define

#ifdef LCD_DRIVER_ST7735S

//
// Macro
//
#define ARG_CHECK

#if defined(LCD_MODULE_Z144SN005)
  #ifdef LCD_LANDSCAPE_MODE
    #ifdef LCD_UPSIDE_DOWN_MODE
      #define X_OFFS	3
      #define Y_OFFS	2
    #else
      #define X_OFFS	1
      #define Y_OFFS	2
    #endif
  #else
    #ifdef LCD_UPSIDE_DOWN_MODE
      #define X_OFFS	2
      #define Y_OFFS	3
    #else
      #define X_OFFS	2
      #define Y_OFFS	1
    #endif
  #endif

#elif defined(LCD_MODULE_Z180SN009) || defined(LCD_MODULE_Z180SN1720)
  #ifdef LCD_LANDSCAPE_MODE
    #ifdef LCD_UPSIDE_DOWN_MODE
      #define X_OFFS	1
      #define Y_OFFS	2
    #else
      #define X_OFFS	1
      #define Y_OFFS	2
    #endif
  #else
    #ifdef LCD_UPSIDE_DOWN_MODE
      #define X_OFFS	2
      #define Y_OFFS	1
    #else
      #define X_OFFS	2
      #define Y_OFFS	1
    #endif
  #endif

#else
  #define X_OFFS	0
  #define Y_OFFS	0
#endif

//
// API
//
void lcd_drawInit(void) {
	lcd_portWriteCmd8(LCD_COMMAND_NOP);
	lcd_portWriteCmd8(LCD_COMMAND_SWRESET);
	lcd_portWait(120);
	lcd_portWriteCmd8(LCD_COMMAND_SLPOUT);
	lcd_portWait(120);

	// Optional setting
#if defined(LCD_MODULE_Z180SN009)
	lcd_portWriteCmdData8(LCD_COMMAND_GAMSET,LCD_DATA_GC2);
#endif
#if defined(LCD_MODULE_ATM0130B3)
	lcd_portWriteCmd8(LCD_COMMAND_INVON);
#endif

	// Required settings
	lcd_portWriteCmdData8(LCD_COMMAND_MADCTL, 0
#if defined(LCD_LANDSCAPE_MODE) && defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MV
#elif defined(LCD_LANDSCAPE_MODE)
		| LCD_DATA_MX
		| LCD_DATA_MV
#elif defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MX
#endif
#if defined(LCD_MODULE_Z144SN005)
	    | LCD_DATA_BGR
#endif
		);
	lcd_portWriteCmdData8(LCD_COMMAND_COLMOD, LCD_DATA_IFPF_16BIT);
	lcd_portWriteCmd8(LCD_COMMAND_DISPON);

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);
	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	short	i;
	short	x2,y2;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif //ARG_CHECK
	x += X_OFFS;
	y += Y_OFFS;
	x2 = x + w - 1;
	y2 = y + h - 1;
	a[0] = x >> 8;
	a[1] = x;
	a[2] = x2 >> 8;
	a[3] = x2;
	lcd_portWriteCmdDatas8(LCD_COMMAND_CASET, sizeof(a), a);
	a[0] = y >> 8;
	a[1] = y;
	a[2] = y2 >> 8;
	a[3] = y2;
	lcd_portWriteCmdDatas8(LCD_COMMAND_RASET, sizeof(a), a);
	lcd_portWriteCmd8(LCD_COMMAND_RAMWR);
	for(i = w * h; i > 0; --i) {
		lcd_portWriteData(color);
	}
	return;
}

//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				// 16bit color / pixel left->right top->down scan
) {
	short	i;
	short	x2,y2;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif //ARG_CHECK
	x += X_OFFS;
	y += Y_OFFS;
	x2 = x + w - 1;
	y2 = y + h - 1;
	a[0] = x >> 8;
	a[1] = x;
	a[2] = x2 >> 8;
	a[3] = x2;
	lcd_portWriteCmdDatas8(LCD_COMMAND_CASET, sizeof(a), a);
	a[0] = y >> 8;
	a[1] = y;
	a[2] = y2 >> 8;
	a[3] = y2;
	lcd_portWriteCmdDatas8(LCD_COMMAND_RASET, sizeof(a), a);

	lcd_portWriteCmd8(LCD_COMMAND_RAMWR);
	for(i = w * h; i > 0; --i) {
		lcd_portWriteData(*image++);
	}

	return;
}

#endif	// LCD_DRIVER_ST7735S

