//
//  lcd_draw_driver_ST7789V2.c	Drawing Layer (Primitive figure elements)
//  LCD module driver/controller IC interface for ST7789V2
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      Controller chip: Sitronix ST7789V2
//          Screen size   : 240x320
//          Color mode    : 65K
//          RGB interface : N/A
//
//      Support LCD Module (LCM):
//          ATM0130B3
//
//      Support function:
//          Primitive figure elements: Rect, Image, Line, Arc, Char, char2
//          LCD_SCREEN_WIDTH     : 240,320
//          LCD_SCREEN_HEIGHT    : 240,320
//          LCD_LANDSCAPE_MODE   : Screen 90deg rotation
//          LCD_UPSIDE_DOWN_MODE : Screen 180deg flip
//
//   Reference:
//      Sitronix Technology Corp.
//      240RGB x 320dot 262K Color with Frame Memory
//      Single-Chip TFT Controller/Driver ST7789V2
//          Datasheet
//              Version     : 1.0
//              Document No.: none
//              Date        : 2016/11
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.07 v0.0  First cording
//


#include "lcd_draw.h"					// API
#include "lcd_draw_port.h"				// Port I/O
#include "lcd_spec_ST7789V2.h"			// Command & parameter define

#ifdef LCD_DRIVER_ST7789V2

//
// Macro
//
#define ARG_CHECK

#if defined(LCD_MODULE_ATM0130B3)
  #ifdef LCD_LANDSCAPE_MODE
    #ifdef LCD_UPSIDE_DOWN_MODE
      #define X_OFFS	(LCD_DRIVER_V - LCD_SCREEN_WIDTH)
      #define Y_OFFS	0
    #else
      #define X_OFFS	0
      #define Y_OFFS	(LCD_DRIVER_H - LCD_SCREEN_HEIGHT)
    #endif
  #else
    #ifdef LCD_UPSIDE_DOWN_MODE
      #define X_OFFS	0
      #define Y_OFFS	(LCD_DRIVER_V - LCD_SCREEN_HEIGHT)
    #else
      #define X_OFFS	0
      #define Y_OFFS	0
    #endif
  #endif
#else
  #define X_OFFS	0
  #define Y_OFFS	0
#endif

//
// API
//
void lcd_drawInit(void) {
	lcd_portWriteCmd8(LCD_COMMAND_NOP);
	lcd_portWriteCmd8(LCD_COMMAND_SWRESET);
	lcd_portWait(120);
	lcd_portWriteCmd8(LCD_COMMAND_SLPOUT);
	lcd_portWait(120);

	// Optional setting
#if defined(LCD_MODULE_ATM0130B3)
	lcd_portWriteCmd8(LCD_COMMAND_INVON);
	{
		unsigned char	b[3] = {LCD_SCREEN_HEIGHT/8, 0, LCD_DATA3_TMG};

		lcd_portWriteCmdDatas8(LCD_COMMAND_GATECTRL, sizeof(b), b);
	}
#endif

	// Required settings
	lcd_portWriteCmdData8(LCD_COMMAND_MADCTL, 0
#if defined(LCD_LANDSCAPE_MODE) && defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MV
#elif defined(LCD_LANDSCAPE_MODE)
		| LCD_DATA_MX
		| LCD_DATA_MV
#elif defined(LCD_UPSIDE_DOWN_MODE)
		| LCD_DATA_MY
		| LCD_DATA_MX
#endif
		);
	lcd_portWriteCmdData8(LCD_COMMAND_COLMOD, LCD_DATA_IFPF_16BIT);
	lcd_portWriteCmd8(LCD_COMMAND_DISPON);

	lcd_drawRect(0,0, LCD_SCREEN_WIDTH,LCD_SCREEN_HEIGHT, LCD_COLOR_BLACK);
	return;
}

//
//	Open
//
void lcd_drawOpen(void)	{
	lcd_portStart();
	lcd_drawInit();
	return;
}

//
//	Rectangle
//
void lcd_drawRect(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	color				//
) {
	int	i;
	int	x2,y2;
	int	hh;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif //ARG_CHECK

	x += X_OFFS;
	y += Y_OFFS;
	hh = h / ((w * h) / LCD_DRIVER_SEND_LIMIT + 1);

	while(hh > 0) {
		x2 = x + w - 1;
		y2 = y + hh - 1;
		LCD_SET_CASET(a, x, x2);
		lcd_portWriteCmdDatas8(LCD_COMMAND_CASET, sizeof(a), a);
		LCD_SET_RASET(a, y, y2);
		lcd_portWriteCmdDatas8(LCD_COMMAND_RASET, sizeof(a), a);
		lcd_portWriteCmd8(LCD_COMMAND_RAMWR);
		for(i = w * hh; i > 0; --i) {
			lcd_portWriteData(color);
		}

		y += hh;
		h -= hh;
		if (h < hh) {
			hh = h;
		}
	}
	return;
}

//
//	Image
//
void lcd_drawImage(
	short			x,					// Upper left position
	short			y,					//
	short			w,					// width
	short			h,					// height
	unsigned short	*image				// 16bit color / pixel left->right top->down scan
) {
	int	i;
	int	x2,y2;
	int	hh;
	unsigned char	a[4];

#ifdef ARG_CHECK
	if (x < 0) {
		w += x;
		x = 0;
	}
	if (y < 0) {
		h += y;
		y = 0;
	}
#endif //ARG_CHECK

	x += X_OFFS;
	y += Y_OFFS;
	hh = h / ((w * h) / LCD_DRIVER_SEND_LIMIT + 1);

	while(hh > 0) {
		x2 = x + w - 1;
		y2 = y + hh - 1;
		LCD_SET_CASET(a, x, x2);
		lcd_portWriteCmdDatas8(LCD_COMMAND_CASET, sizeof(a), a);
		LCD_SET_RASET(a, y, y2);
		lcd_portWriteCmdDatas8(LCD_COMMAND_RASET, sizeof(a), a);

		lcd_portWriteCmd8(LCD_COMMAND_RAMWR);
		for(i = w * hh; i > 0; --i) {
			lcd_portWriteData(*image++);
		}

		y += hh;
		h -= hh;
		if (h < hh) {
			hh = h;
		}
	}

	return;
}

#endif	// LCD_DRIVER_ST7789V2



