//
//  lcd_draw_port_ESP32.h	Transport layer
//  Physical transport interface for Espressif Systems ESP32
//  Copyright(C)2023 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//		Command and data transfer subroutines.
//
//      LCD module:
//		Not depend, interface is SPI(4wire) only
//
//      Host device:
//          ESP32 Series
//
//      IDE:
//          esp-idf v5.1
//
//      Library:
//          esp-idf v5.1
//
//	Settings
//		idf.py menuconfig -> Component config > Graphic LCD module driver configuration
//		GPIO pin CS
//		GPIO pin SCK
//		GPIO pin SDA
//		GPIO pin DC
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2023.02.26 v0.0  First cording
//

#include "lcd_draw_port.h"

#include "driver/gpio.h"
#include "freertos/FreeRTOS.h"
#include "freertos/task.h"

//
// General
//
void lcd_portStart(void) {
	gpio_config_t	c = {
		.pin_bit_mask	= (1 << CONFIG_LCD_MODULE_GPIO_CS)
				| (1 << CONFIG_LCD_MODULE_GPIO_SCK)
				| (1 << CONFIG_LCD_MODULE_GPIO_SDA)
				| (1 << CONFIG_LCD_MODULE_GPIO_DC)
				,
		.mode		= GPIO_MODE_OUTPUT,
		.pull_up_en	= GPIO_PULLUP_DISABLE,
		.pull_down_en	= GPIO_PULLDOWN_DISABLE,
		.intr_type	= GPIO_INTR_DISABLE,
	};

	ESP_ERROR_CHECK(gpio_config(&c));
	ESP_ERROR_CHECK(gpio_set_level(CONFIG_LCD_MODULE_GPIO_CS, 1));
	ESP_ERROR_CHECK(gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 0));
	return;
}

void lcd_portStop(void) {
	ESP_ERROR_CHECK(gpio_reset_pin(CONFIG_LCD_MODULE_GPIO_CS));
	ESP_ERROR_CHECK(gpio_reset_pin(CONFIG_LCD_MODULE_GPIO_SCK));
	ESP_ERROR_CHECK(gpio_reset_pin(CONFIG_LCD_MODULE_GPIO_SDA));
	ESP_ERROR_CHECK(gpio_reset_pin(CONFIG_LCD_MODULE_GPIO_DC));
	return;
}

void lcd_portWait(int ms) {
	vTaskDelay(ms / portTICK_PERIOD_MS);
	return;
}

__attribute__((weak))					// overridable
void lcd_portWrite(int rs, unsigned short data) {
	unsigned char	d[2];

	d[0] = data >> 8;
	d[1] = data;
	lcd_portWrites8(rs, sizeof(d), d);
	return;
}


//---------------------------------------------------------------------
//  SPI 4-Wire interface
//---------------------------------------------------------------------
//
// Software mode
//
void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	gpio_set_level(CONFIG_LCD_MODULE_GPIO_DC, rs);
	gpio_set_level(CONFIG_LCD_MODULE_GPIO_CS, 0);
	for(; len > 0; --len) {
		int	d;
		int	i;

		d = *data;
		for(i = 8; i > 0; --i) {
			gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 0);
			if (d & 0x80) {
				gpio_set_level(CONFIG_LCD_MODULE_GPIO_SDA, 1);
			} else {
				gpio_set_level(CONFIG_LCD_MODULE_GPIO_SDA, 0);
			}
			gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 1);
			d = d << 1;
		}
		data++;
	}
	gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 0);
	gpio_set_level(CONFIG_LCD_MODULE_GPIO_CS, 1);
//	vTaskDelay(1);	// debug 10ms
	return;
}

#ifdef DELETE
//---------------------------------------------------------------------
//  SPI 3-Wire interface
//---------------------------------------------------------------------
//
// Software mode
//
void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	gpio_set_level(CONFIG_LCD_MODULE_GPIO_CS, 0);
	for(; len > 0; --len) {
		int	d;
		int	i;

		gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 0);
		if (rs) {
			gpio_set_level(CONFIG_LCD_MODULE_GPIO_SDA, 1);
		} else {
			gpio_set_level(CONFIG_LCD_MODULE_GPIO_SDA, 0);
		}
		gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 1);

		d = *data;
		for(i = 8; i > 0; --i) {
			gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 0);
			if (d & 0x80) {
				gpio_set_level(CONFIG_LCD_MODULE_GPIO_SDA, 1);
			} else {
				gpio_set_level(CONFIG_LCD_MODULE_GPIO_SDA, 0);
			}
			gpio_set_level(CONFIG_LCD_MODULE_GPIO_SCK, 1);
			d = d << 1;
		}
		data++;
	}
	gpio_set_level(CONFIG_LCD_MODULE_GPIO_CS, 1);
	return;
}
#endif
