//
//  lcd_draw_port_STM32CubeF4.h	Transport layer
//  Physical transport interface for STM32Cube
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//      Command and data transfer subroutines.
//
//      LCD module: General
//
//      Host device:
//          STM32F4 Series
//          NUCLEO_F446RE    :	STM32 NUCLEO64 F446RE
//          STM32F4DISCOVERY :	STM32F4-DISCOVERY
//      IDE:
//          STM32CubeMX + STM32CubeIDE
//      Library:
//          STM32Cube F4 HAL
//
//	Settings
//		STM32CubeMX Pinout & Configuration -> System Core -> GPIO
//
//      Pin   output     GPIO mode          Pull-up             Maximum    User Label
//      Name  level                         Pull-down        output speed
//      ----  ------  ----------------  -------------------  ------------  ------------------------ Parallel 8 or 16bit
//      P**   High    Output Push Pull  No pull-up and down       High     LCM_CS
//      P**   High    Output Push Pull  No pull-up and down       High     LCM_RS
//      P**   High    Output Push Pull  No pull-up and down       High     LCM_WR
//      P*0    *      Output Push Pull  No pull-up and down        *       LCM_D0 or LCM_D0X,LCM_D0Y
//      P*1    *      Output Push Pull  No pull-up and down        *       LCM_D1 or LCM_D1X,LCM_D1Y
//      P*2    *      Output Push Pull  No pull-up and down        *       LCM_D2 or LCM_D2X,LCM_D2Y
//       :     :               :                  :                :        :
//      P*15   *      Output Push Pull  No pull-up and down        *       LCM_D15 or LCM_D15X,LCM_D15Y
//      ----  ------  ----------------  -------------------  ------------  ------------------------ SPI 4-wire
//      P**   High    Output Push Pull  No pull-up and down       High     LCM_CS
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_SCK
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_SDA
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_DC
//      ----  ------  ----------------  -------------------  ------------  ------------------------ SPI 3-wire
//      P**   High    Output Push Pull  No pull-up and down       High     LCM_CS
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_SCK
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_SDA
//      ----  ------  ----------------  -------------------  ------------  ------------------------ SPI Start byte
//      P**   High    Output Push Pull  No pull-up and down       High     LCM_CS
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_SCK
//      P**    *      Input             Pull-up                   High     LCM_SDI
//      P**    *      Output Push Pull  No pull-up and down       High     LCM_SDO
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.19 v0.0  First cording
//

#include "lcd_draw_port.h"

//
//	Config
//                              STM32CubeMX pin setting "User Label"
// Parallel 8080 bus 16-bit		LCM_CS, LCM_RS, LCM_WR, LCM_D0..D15 or LCM_D*X and Y(split ports)
// Parallel 8080 bus 8-bit		LCM_CS, LCM_RS, LCM_WR, LCM_D0..D7
// SPI 4-Wire					LCM_CS, LCM_SCK, LCM_SDA, LCM_DC
//     Use SPI peripheral in STM32F4 -> define "LCD_DRAW_SPI_CB=hspi*"
//         STM32CubeMX SPIx setting: Mode=Half-Duplex, Baud_Rate < 15MHz, others default
//     If not defined is "software emulated SPI"
// SPI 3-Wire					LCM_CS, LCM_SCK, LCM_SDA
// SPI Start byte method		LCM_CS, LCM_SCK, LCM_SDI, LCM_SDO
//


//---------------------------------------------------------------------
//
//						STM32Cube HAL Drivers
//
//---------------------------------------------------------------------
#ifdef USE_HAL_DRIVER
#include <stm32f4xx_hal.h>

//
// HAL speed up cheat
//
#define HAL_GPIO_SetPin(port, pin)			port->BSRR = pin
#define HAL_GPIO_ResetPin(port, pin)		port->BSRR = (uint32_t)pin << 16U
#define HAL_GPIO_Write(port, pins, data)	port->ODR = (port->ODR & ~pins) | (data & pins)
#define HAL_GPIO_Read(port, pins)			(port->IDR & pins)

//
// Pin assign
//
#include "main.h"						// Use a "STM32CubeMX" settings

//
// General
//
void lcd_portStart(void) {
	// Setting is CubeMX auto generating codes
	return;
}

void lcd_portStop(void) {
	// Setting is CubeMX auto generating codes
	return;
}

void lcd_portWait(int ms) {
	uint32_t	w;

	w = HAL_GetTick() + ms;
	while(w > HAL_GetTick());			// CPU block
	return;
}

//void lcd_portPeriod(void) {
//	HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
//	return;
//}

__attribute__((weak))					// overridable
void lcd_portWrite(int rs, unsigned short data) {
	unsigned char	d[2];

	d[0] = data >> 8;
	d[1] = data;
	lcd_portWrites8(rs, sizeof(d), d);
	return;
}


#if defined(LCM_SCK_Pin) && defined(LCM_SDA_Pin) && defined(LCM_DC_Pin)
//---------------------------------------------------------------------
//  SPI 4-Wire interface
//---------------------------------------------------------------------
#ifdef LCD_DRAW_SPI_CB				// Control block name "hspiX" in main.h
//
// Hardware mode
//
extern SPI_HandleTypeDef LCD_DRAW_SPI_CB;

void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	HAL_StatusTypeDef s;

	HAL_GPIO_WritePin(LCM_DC_GPIO_Port, LCM_DC_Pin, rs);
	HAL_GPIO_ResetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	s = HAL_SPI_Transmit(&LCD_DRAW_SPI_CB, (uint8_t *)data, len, 1);
	if (s != HAL_OK) {
		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);
	}
	HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	return;
}

#else
//
// Software mode
//
void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	HAL_GPIO_WritePin(LCM_DC_GPIO_Port, LCM_DC_Pin, rs);
	HAL_GPIO_ResetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	for(; len > 0; --len) {
		int	d;
		int	i;

		d = *data;
		for(i = 8; i > 0; --i) {
			HAL_GPIO_ResetPin(LCM_SCK_GPIO_Port, LCM_SCK_Pin);
			if (d & 0x80) {
				HAL_GPIO_SetPin(LCM_SDA_GPIO_Port, LCM_SDA_Pin);
			} else {
				HAL_GPIO_ResetPin(LCM_SDA_GPIO_Port, LCM_SDA_Pin);
			}
			HAL_GPIO_SetPin(LCM_SCK_GPIO_Port, LCM_SCK_Pin);
			d = d << 1;
		}
		data++;
	}
	HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	return;
}
#endif

#elif defined(LCM_SCK_Pin) && defined(LCM_SDA_Pin)
//---------------------------------------------------------------------
//  SPI 3-Wire interface
//---------------------------------------------------------------------
//
// Software mode
//
void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	HAL_GPIO_ResetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	for(; len > 0; --len) {
		int	d;
		int	i;

		HAL_GPIO_ResetPin(LCM_SCK_GPIO_Port, LCM_SCK_Pin);
		if (rs) {
			HAL_GPIO_SetPin(LCM_SDA_GPIO_Port, LCM_SDA_Pin);
		} else {
			HAL_GPIO_ResetPin(LCM_SDA_GPIO_Port, LCM_SDA_Pin);
		}
		HAL_GPIO_SetPin(LCM_SCK_GPIO_Port, LCM_SCK_Pin);

		d = *data;
		for(i = 8; i > 0; --i) {
			HAL_GPIO_ResetPin(LCM_SCK_GPIO_Port, LCM_SCK_Pin);
			if (d & 0x80) {
				HAL_GPIO_SetPin(LCM_SDA_GPIO_Port, LCM_SDA_Pin);
			} else {
				HAL_GPIO_ResetPin(LCM_SDA_GPIO_Port, LCM_SDA_Pin);
			}
			HAL_GPIO_SetPin(LCM_SCK_GPIO_Port, LCM_SCK_Pin);
			d = d << 1;
		}
		data++;
	}
	HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	return;
}

#elif defined(LCM_SCK_Pin) && defined(LCM_SDI_Pin) && defined(LCM_SDO_Pin)
//---------------------------------------------------------------------
//  SPI interface and start byte method
//---------------------------------------------------------------------

//---------------------------------------------------------------------
//  i8080 bus and 8 or 16bits parallel interface
//---------------------------------------------------------------------
//
// Data bus is "16bit single port"
//
#elif defined(LCM_D15_Pin)

#define LCM_D_Pins	(LCM_D0_Pin | LCM_D1_Pin | LCM_D2_Pin | LCM_D3_Pin | LCM_D4_Pin | LCM_D5_Pin | LCM_D6_Pin | LCM_D7_Pin \
		           | LCM_D8_Pin | LCM_D9_Pin | LCM_D10_Pin | LCM_D11_Pin | LCM_D12_Pin | LCM_D13_Pin | LCM_D14_Pin | LCM_D15_Pin)

void lcd_portWrite(int rs, unsigned short data) {
	HAL_GPIO_WritePin(LCM_RS_GPIO_Port, LCM_RS_Pin, rs);
	HAL_GPIO_ResetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	HAL_GPIO_Write(LCM_D0_GPIO_Port, LCM_D_Pins, data);
	HAL_GPIO_ResetPin(LCM_WR_GPIO_Port, LCM_WR_Pin);
	HAL_GPIO_SetPin  (LCM_WR_GPIO_Port, LCM_WR_Pin);
	HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	return;
}

void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	for(; len > 0; --len) {
		lcd_portWrite(rs, (unsigned short)*data++);		// D15..8 = 0b00000000
	}
	return;
}

//
// Data bus is "16bit separated two ports"
//
#elif defined(LCM_D15X_Pin) || defined(LCM_D15Y_Pin)

#ifdef LCM_D0X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D0X_GPIO_Port
  #endif
#else
  #define LCM_D0X_Pin	0
#endif

#ifdef LCM_D1X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D1X_GPIO_Port
  #endif
#else
  #define LCM_D1X_Pin	0
#endif

#ifdef LCM_D2X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D2X_GPIO_Port
  #endif
#else
  #define LCM_D2X_Pin	0
#endif

#ifdef LCM_D3X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D3X_GPIO_Port
  #endif
#else
  #define LCM_D3X_Pin	0
#endif

#ifdef LCM_D4X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D4X_GPIO_Port
  #endif
#else
  #define LCM_D4X_Pin	0
#endif

#ifdef LCM_D5X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D5X_GPIO_Port
  #endif
#else
  #define LCM_D5X_Pin	0
#endif

#ifdef LCM_D6X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D6X_GPIO_Port
  #endif
#else
  #define LCM_D6X_Pin	0
#endif

#ifdef LCM_D7X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D7X_GPIO_Port
  #endif
#else
  #define LCM_D7X_Pin	0
#endif

#ifdef LCM_D8X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D8X_GPIO_Port
  #endif
#else
  #define LCM_D8X_Pin	0
#endif

#ifdef LCM_D9X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D9X_GPIO_Port
  #endif
#else
  #define LCM_D9X_Pin	0
#endif

#ifdef LCM_D10X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D10X_GPIO_Port
  #endif
#else
  #define LCM_D10X_Pin	0
#endif

#ifdef LCM_D11X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D11X_GPIO_Port
  #endif
#else
  #define LCM_D11X_Pin	0
#endif

#ifdef LCM_D12X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D12X_GPIO_Port
  #endif
#else
  #define LCM_D12X_Pin	0
#endif

#ifdef LCM_D13X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D13X_GPIO_Port
  #endif
#else
  #define LCM_D13X_Pin	0
#endif

#ifdef LCM_D14X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D14X_GPIO_Port
  #endif
#else
  #define LCM_D14X_Pin	0
#endif

#ifdef LCM_D15X_Pin
  #ifndef LCM_DX_Port
    #define LCM_DX_Port	LCM_D15X_GPIO_Port
  #endif
#else
  #define LCM_D15X_Pin	0
#endif

#define LCM_DX_Pins	(LCM_D0X_Pin | LCM_D1X_Pin | LCM_D2X_Pin | LCM_D3X_Pin | LCM_D4X_Pin | LCM_D5X_Pin | LCM_D6X_Pin | LCM_D7X_Pin \
		           | LCM_D8X_Pin | LCM_D9X_Pin | LCM_D10X_Pin | LCM_D11X_Pin | LCM_D12X_Pin | LCM_D13X_Pin | LCM_D14X_Pin | LCM_D15X_Pin)

#ifdef LCM_D0Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D0Y_GPIO_Port
  #endif
#else
  #define LCM_D0Y_Pin	0
#endif

#ifdef LCM_D1Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D1Y_GPIO_Port
  #endif
#else
  #define LCM_D1Y_Pin	0
#endif

#ifdef LCM_D2Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D2Y_GPIO_Port
  #endif
#else
  #define LCM_D2Y_Pin	0
#endif

#ifdef LCM_D3Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D3Y_GPIO_Port
  #endif
#else
  #define LCM_D3Y_Pin	0
#endif

#ifdef LCM_D4Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D4Y_GPIO_Port
  #endif
#else
  #define LCM_D4Y_Pin	0
#endif

#ifdef LCM_D5Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D5Y_GPIO_Port
  #endif
#else
  #define LCM_D5Y_Pin	0
#endif

#ifdef LCM_D6Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D6Y_GPIO_Port
  #endif
#else
  #define LCM_D6Y_Pin	0
#endif

#ifdef LCM_D7Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D7Y_GPIO_Port
  #endif
#else
  #define LCM_D7Y_Pin	0
#endif

#ifdef LCM_D8Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D8Y_GPIO_Port
  #endif
#else
  #define LCM_D8Y_Pin	0
#endif

#ifdef LCM_D9Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D9Y_GPIO_Port
  #endif
#else
  #define LCM_D9Y_Pin	0
#endif

#ifdef LCM_D10Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D10Y_GPIO_Port
  #endif
#else
  #define LCM_D10Y_Pin	0
#endif

#ifdef LCM_D11Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D11Y_GPIO_Port
  #endif
#else
  #define LCM_D11Y_Pin	0
#endif

#ifdef LCM_D12Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D12Y_GPIO_Port
  #endif
#else
  #define LCM_D12Y_Pin	0
#endif

#ifdef LCM_D13Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D13Y_GPIO_Port
  #endif
#else
  #define LCM_D13Y_Pin	0
#endif

#ifdef LCM_D14Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D14Y_GPIO_Port
  #endif
#else
  #define LCM_D14Y_Pin	0
#endif

#ifdef LCM_D15Y_Pin
  #ifndef LCM_DY_Port
    #define LCM_DY_Port	LCM_D15Y_GPIO_Port
  #endif
#else
  #define LCM_D15Y_Pin	0
#endif

#define LCM_DY_Pins	(LCM_D0Y_Pin | LCM_D1Y_Pin | LCM_D2Y_Pin | LCM_D3Y_Pin | LCM_D4Y_Pin | LCM_D5Y_Pin | LCM_D6Y_Pin | LCM_D7Y_Pin \
		           | LCM_D8Y_Pin | LCM_D9Y_Pin | LCM_D10Y_Pin | LCM_D11Y_Pin | LCM_D12Y_Pin | LCM_D13Y_Pin | LCM_D14Y_Pin | LCM_D15Y_Pin)

void lcd_portWrite(int rs, unsigned short data) {
	HAL_GPIO_Write(LCM_DX_Port, LCM_DX_Pins, data);
	HAL_GPIO_Write(LCM_DY_Port, LCM_DY_Pins, data);

	HAL_GPIO_WritePin(LCM_RS_GPIO_Port, LCM_RS_Pin, rs);
	HAL_GPIO_ResetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);

	HAL_GPIO_ResetPin(LCM_WR_GPIO_Port, LCM_WR_Pin);
	HAL_GPIO_SetPin  (LCM_WR_GPIO_Port, LCM_WR_Pin);
	HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
	return;
}

void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	for(; len > 0; --len) {
		lcd_portWrite(rs, (unsigned short)*data++);		// D15..8 = 0b00000000
	}
	return;
}

#elif defined(LCM_D7_Pin)
//---------------------------------------------------------------------
//  i8080 bus and 8bits parallel interface
//---------------------------------------------------------------------
#define LCM_D_Pins	(LCM_D0_Pin | LCM_D1_Pin | LCM_D2_Pin | LCM_D3_Pin | LCM_D4_Pin | LCM_D5_Pin | LCM_D6_Pin | LCM_D7_Pin)

void lcd_portWrites8(int rs, size_t len, unsigned char *data) {
	HAL_GPIO_WritePin(LCM_RS_GPIO_Port, LCM_RS_Pin, rs);
	for(; len > 0; --len) {
		HAL_GPIO_Write(LCM_D0_GPIO_Port, LCM_D_Pins, *data);
		HAL_GPIO_ResetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
		HAL_GPIO_ResetPin(LCM_WR_GPIO_Port, LCM_WR_Pin);
		HAL_GPIO_SetPin  (LCM_WR_GPIO_Port, LCM_WR_Pin);
		HAL_GPIO_SetPin(LCM_CS_GPIO_Port, LCM_CS_Pin);
		data++;
	}
	return;
}

#else
//---------------------------------------------------------------------
// Unknown interface
//---------------------------------------------------------------------
#error "Not supported port interface"
#endif


//---------------------------------------------------------------------
//
//						Other driver
//
//---------------------------------------------------------------------
#else	// USE_HAL_DRIVER				//

#error "Not supported HAL library"

#endif	// USE_HAL_DRIVER				//

