//
//  lcd_spec_ST7789V2.h	Sitronix Single-Chip TFT Controller/Driver ST7789V2 command code
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//   Reference:
//      Sitronix Technology Corp.
//      ST7789V2 - single-chip controller/driver for 262K-color, graphic type TFT-LCD 240RGB x 320
//          Datasheet
//              Version     : 1.0
//              Document No.: none
//              Date        : 2016/11
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.16 v0.0  First cording
//

#ifndef LCD_SPEC_ST7789V2_H_
#define LCD_SPEC_ST7789V2_H_

//
// Const
//
#define LCD_DRIVER_H			240		// RAM size
#define LCD_DRIVER_V			320		//
#define LCD_DRIVER_SEND_LIMIT	32768	// Color data

//
// System Function Command
//
#define LCD_COMMAND_NOP			0x00	// No operation

#define LCD_COMMAND_SWRESET		0x01	// Software reset

#define LCD_COMMAND_SLPIN		0x10	// Sleep in & booster off

#define LCD_COMMAND_SLPOUT		0x11	// Sleep out & booster on

#define LCD_COMMAND_PTLON		0x12	// Partial mode on

#define LCD_COMMAND_NORON		0x13	// Partial off (normal)

#define LCD_COMMAND_INVOFF		0x20	// Display inversion off (normal)

#define LCD_COMMAND_INVON		0x21	// Display inversion on

#define LCD_COMMAND_GAMSET		0x26	// Gamma curve select (1byte)
#define   LCD_DATA_GC0				0x01	//
#define   LCD_DATA_GC1				0x02	//
#define   LCD_DATA_GC2				0x04	//
#define   LCD_DATA_GC3				0x08	//

#define LCD_COMMAND_DISPOFF		0x28	// Display off

#define LCD_COMMAND_DISPON		0x29	// Display on

#define LCD_COMMAND_CASET		0x2A	// Column address set (4byte)
#define   LCD_SET_CASET(b,xs, xe)		b[0]=(xs)>>8; b[1]=(xs); b[2]=(xe)>>8; b[3]=(xe)

#define LCD_COMMAND_RASET		0x2B	// Row address set (4byte)
#define   LCD_SET_RASET(b,ys,ye)		b[0]=(ys)>>8; b[1]=(ys); b[2]=(ye)>>8; b[3]=(ye)

#define LCD_COMMAND_RAMWR		0x2C	// Memory write (n word)

#define LCD_COMMAND_RGBSET		0x2D	// LUT for indexed color (128byte)

#define LCD_COMMAND_PTLAR		0x30	// Partial start/end address set (4byte)
#define   LCD_SET_PTLAR(b,psl,pel)		b[0]=(psl)>>8; b[1]=(psl); b[2]=(pel)>>8; b[3]=(pel)

#define LCD_COMMAND_VSCRDEF		0x33	// Vertical scrolling definition (6byte)
#define   LCD_SET_VSCRDEF(b,tfa,vsa,bfa)	b[0]=(tfa)>>8; b[1]=(tfa); b[2]=(vsa)>>8; b[3]=(vsa); b[4]=(bfa)>>8; b[5]=(bfa)

#define LCD_COMMAND_TEOFF		0x34	// Tearing effect line off

#define LCD_COMMAND_TEON		0x35	// Tearing effect mode set & on (1byte)
#define  LCD_DATA_TEM				(1<<0)	// The Tearing Effect output Line consists of both V-Blanking and H-Blanking information

#define LCD_COMMAND_MADCTL		0x36	// Memory data access control (1byte)
#define   LCD_DATA_MY				(1<<7)	// Row Address Order
#define   LCD_DATA_MX				(1<<6)	// Column Address Order
#define   LCD_DATA_MV				(1<<5)	// Row/Column Exchange
#define   LCD_DATA_ML				(0<<4)	// Vertical Refresh Order
#define   LCD_DATA_ML_Top_Bottom	(0<<4)	//
#define   LCD_DATA_ML_Bottom_Top	(1<<4)	//
#define   LCD_DATA_RGB				(0<<3)	// RGB-BGR ORDER
#define   LCD_DATA_BGR				(1<<3)	// RGB-BGR ORDER
#define   LCD_DATA_MH				(1<<2)	// Horizontal Refresh Order
#define   LCD_DATA_MH_Left_Right	(0<<2)	//
#define   LCD_DATA_MH_Right_Left	(1<<2)	//

#define LCD_COMMAND_VSCRSADD	0x37	// Vertical scrolling start address (2byte)
#define   LCD_SET_VSCRSADD(b,vsp)		b[0]=(vsp)>>8; b[1]=(vsp)

#define LCD_COMMAND_IDMOFF		0x38	// Idle mode off

#define LCD_COMMAND_IDMON		0x39	// Idle mode on

#define LCD_COMMAND_COLMOD		0x3A	// Interface pixel format (1byte)
#define   LCD_DATA_VIPF(n)			((n)<<4)	// RGB Interface Color Format
#define   LCD_DATA_VIPF_16BIT		LCD_DATA_VIPF(0b101)	// 65K
#define   LCD_DATA_VIPF_18BIT		LCD_DATA_VIPF(0b110)	// 262K
#define   LCD_DATA_IFPF(n)			((n)<<0)	// Control interface color format
#define   LCD_DATA_IFPF_12BIT		LCD_DATA_IFPF(0b011)	// 4K
#define   LCD_DATA_IFPF_16BIT		LCD_DATA_IFPF(0b101)	// 65K
#define   LCD_DATA_IFPF_18BIT		LCD_DATA_IFPF(0b110)	// 262K
#define   LCD_DATA_IFPF_16M			LCD_DATA_IFPF(0b111)	// 16M truncated

#define LCD_COMMAND_RAMWRC		0x3C	// Write data continue
#define LCD_COMMAND_WRMEMC		0x3C	// Write Memory Continue

#define LCD_COMMAND_TESCAN		0x44	// Set tear scanline (2byte)
#define   LCD_SET_TESCAN(b,n)			b[0]=(n)>>8; b[1]=(n)

#define LCD_COMMAND_WRDISBV		0x51	// Write display brightness (1byte)

#define LCD_COMMAND_WRCTRLD		0x53	// Write CTRL display (1byte)
#define   LCD_DATA_BCTRL			(1<<5)	// Brightness Control Block On/Off
#define   LCD_DATA_DD				(1<<3)	// Display Dimming
#define   LCD_DATA_BL				(1<<2)	// Backlight Control On/Off

#define LCD_COMMAND_WRCACE		0x55	// Write content adaptive brightness control and Color enhancemnet (1byte)
#define   LCD_DATA_CECTRL			(1<<7)		// Color Enhancement Control
#define   LCD_DATA_CE(n)			((n)<<4)	// Color enhancement level
#define   LCD_DATA_CE_Low			LCD_DATA_CE(0)
#define   LCD_DATA_CE_Medium		LCD_DATA_CE(1)
#define   LCD_DATA_CE_High			LCD_DATA_CE(3)
#define   LCD_DATA_C(n)				((n)<<0)	// Function
#define   LCD_DATA_C_Off			LCD_DATA_C(0)
#define   LCD_DATA_C_UIMode			LCD_DATA_C(1)
#define   LCD_DATA_C_StillPicture	LCD_DATA_C(2)
#define   LCD_DATA_C_MovingImage	LCD_DATA_C(3)

#define LCD_COMMAND_WRCABCMB	0x5e	// Write CABC minimum brightness (1byte)


//
// System Function Command 2
//

#define LCD_COMMAND_RAMCTRL		0xB0	// RAM Control (2byte)
#define   LCD_DATA1_DM(n)			((n)<<0)	// Display operation selection
#define   LCD_DATA1_DM_MCU			LCD_DATA1_DM(0b00)
#define   LCD_DATA1_DM_RGB			LCD_DATA1_DM(0b01)
#define   LCD_DATA1_DM_VSYNC		LCD_DATA1_DM(0b10)
#define   LCD_DATA1_RM				(1<<4)		// ram access selection
#define   LCD_DATA1_RM_RGB			(1<<4)		//
#define   LCD_DATA1_RM_MCU			(0<<4)		//
#define   LCD_DATA2_EPF(n)			((n)<<4)	// Data translate of 65k and 4k to frame data
#define   LCD_DATA2_ENDIAN			(1<<3)		//
#define   LCD_DATA2_ENDIAN_LSB		(1<<3)		// Little Endian
#define   LCD_DATA2_ENDIAN_MSB		(0<<3)		// Normal
#define   LCD_DATA2_RIM				(1<<2)		// Specify RGB interface bus width
#define   LCD_DATA2_RIM_6BIT		(1<<2)		//
#define   LCD_DATA2_RIM_18BIT		(0<<2)		//
#define   LCD_DATA2_MDT(n)			((n)<<0)	// Method of pixel data transfer

#define LCD_COMMAND_RGBCTRL		0xB1	// RGB Interface Control (3byte)
#define   LCD_DATA1_WO				(1<<7)		// Direct RGB mod
#define   LCD_DATA1_RCM(n)			((n)<<5)	// RGB I/F enable mode selection
#define   LCD_DATA1_VSPL			(1<<3)		// Sets the signal polarity of the VSYNC pin
#define   LCD_DATA1_HSPL			(1<<2)		// Sets the signal polarity of the HSYNC pin
#define   LCD_DATA1_DPL				(1<<1)		// Sets the signal polarity of the DOTCLK pin
#define   LCD_DATA1_EPL				(1<<0)		// Sets the signal polarity of the ENABLE pin
#define   LCD_SET_RGBCTRL(b,vbp,hbp)	b[1]=vbp; b[2]=hbp

#define LCD_COMMAND_PORCTRL		0xB2	// Porch Control (5byte)
#define   LCD_SET_PORCTRL(b,bpa,fpa,psen,bpb,fpb,bpc,fpc)	\
										b[0]=(bpa); \
										b[1]=(fpa); \
										b[2]=(psen); \
										b[3]=(((bpb)<<4) | (fpb)); \
										b[4]=(((bpc)<<4) | (fpc))

#define LCD_COMMAND_FRMCTR1		0xB3	// Frame Rate Control 1 (3byte)
#define   LCD_SET_FRMCTR1(b,frsen,div,nlb,rtnb,nlc,rtnc)	\
										b[0]=(frsen)<<4 | (div); \
										b[1]=(nlb)<<5 | (rtnb); \
										b[2]=(nlc)<<5 | (rtnc)

#define LCD_COMMAND_PARCTRL		0xb5	// Partial Control (1byte)
#define   LCD_DATA_NDL				(1<<7)		// Source output level in non display area in partial mode
#define   LCD_DATA_PTGISC			(1<<4)		// Non display area scan mode
#define   LCD_DATA_ISC(n)			((n)<<0)	// non-display area scan cycle selection

#define LCD_COMMAND_GCTRL		0xb7	// Gate Control (1byte)
#define   LCD_DATA_VGHS(n)			((n)<<4)	// VGH Setting
#define   LCD_DATA_VGLS(n)			((n)<<0)	// VGL Setting

#define LCD_COMMAND_GTADJ		0xb8	// Gate on timing adjustment (4byte)
#define   LCD_SET_GTADJ(b,gta,gofr,gof)	\
										b[0]=0x2a; \
										b[1]=0x2b; \
										b[2]=gta; \
										b[3]=(gofr)<<4 | (gof)

#define LCD_COMMAND_DGMEN		0xba	// Digital Gamma Enable (1byte)
#define   LCD_DATA_DGMEN			(1<<2)		// enable digital gamma

#define LCD_COMMAND_VCOMS		0xbb	// VCOM Setting (1byte)

#define LCD_COMMAND_POWSAVE		0xbc	// Power saving mode (1byte)
#define   LCD_DATA_POWSAVE			0b11101100	// default
#define   LCD_DATA_NS				(1<<1)		// Power save for normal mode
#define   LCD_DATA_IS				(1<<0)		// Power save for Idle mode

#define LCD_COMMAND_DLPOFFSAVE	0xbd	// Power off power save (1byte)
#define   LCD_DATA_DOFSAVE			(1<<0)		// Power save for display off mode

#define LCD_COMMAND_LCMCTRL		0xc0	// LCM control (1byte)
#define   LCD_DATA_XMY				(1<<6)		// XOR MY setting in command 36h
#define   LCD_DATA_XBGR				(1<<5)		// XOR RGB setting in command 36h
#define   LCD_DATA_XINV				(1<<4)		// XOR inverse setting in command 21h
#define   LCD_DATA_XMX				(1<<3)		// XOR MX setting in command 36h
#define   LCD_DATA_XMH				(1<<2)		// this bit can reverse source output order and only support for RGB interface without RAM mode
#define   LCD_DATA_XMV				(1<<1)		// XOR MV setting in command 36h
#define   LCD_DATA_XGS				(1<<0)		// XOR GS setting in command E4h

#define LCD_COMMAND_IDSET		0xc1	// ID Setting (3byte)
#define   LCD_SET_IDSET(b,id1,id2,id3)	\
										b[0]=id1; \
										b[1]=id2; \
										b[2]=id3;

#define LCD_COMMAND_VDVVRHEN	0xc2	// VDV and VRH Command Enable (2byte)
#define   LCD_SET_VDVVRHEN(b,cmden)	\
										b[0]=(cmden); \
										b[1]=0xff;

#define LCD_COMMAND_VRHS		0xc3	// VRH Set (1byte)

#define LCD_COMMAND_VDVSET		0xc4	// VDV Settting (1byte)

#define LCD_COMMAND_VCMOFSET	0xc5	// VCOM Offset Set (1byte)

#define LCD_COMMAND_FRCTR2		0xc6	// VRH Set (1byte)
#define   LCD_DATA_NLA(n)			((n)<<5)	// Inversion selection in normal mode
#define   LCD_DATA_RTNA(n)			((n)<<0)	// Frame Rate parameter

#define LCD_COMMAND_CABCCTRL	0xc7	// CABC Control (1byte)
#define   LCD_DATA_LEDONREV			(1<<3)		// Reverse the status of LED_ON
#define   LCD_DATA_DPOFPWM			(1<<2)		// initial state control of LEDPWM
#define   LCD_DATA_PWMFIX			(1<<1)		// LEDPWM fix control
#define   LCD_DATA_PWMPOL			(1<<0)		// LEDPWM polarity control

#define LCD_COMMAND_REGSEL1		0xc8	// Register value selection1 (1byte)
#define   LCD_DATA_REGSEL1			0x08		//

#define LCD_COMMAND_REGSEL2		0xca	// Register value selection2 (1byte)
#define   LCD_DATA_REGSEL2			0x0f		//

#define LCD_COMMAND_PWMFRSEL	0xcc	// PWM Frequency Selection (1byte)
#define   LCD_DATA_CS(n)			((n)<<3)	//
#define   LCD_DATA_CLK(n)			((n)<<0)	//

#define LCD_COMMAND_PWCTRL1		0xd0	// Power control 1 (1byte)
#define   LCD_DATA1_PWCTRL1			0xa4		//
#define   LCD_DATA2_AVDD(n)			((n)<<6)	// 0=6.4V 1=6.6V 2=6.8V
#define   LCD_DATA2_AVCL(n)			((n)<<4)	// 0=-4.4V 1=-4.6V 2=-4.8V 3=-5.0
#define   LCD_DATA2_VDS(n)			((n)<<0)	// 0=2.19V 1=2.3V 2=2.4V 3=2.51V

#define LCD_COMMAND_VAPVANEN	0xd2	// nable VAP/VAN signal output (1byte)
#define   LCD_DATA_VAPVANEN			0x4c	//

#define LCD_COMMAND_CMD2EN		0xdf	// Command 2 Enable (1byte)
#define   LCD_SET_CMD2EN(b,en)			b[0]=0x5a; \
										b[1]=0x69; \
										b[2]=0x02; \
										b[3]=(en);

#define LCD_COMMAND_PVGAMCTRL	0xe0	// Positive Voltage Gamma Control (14byte)
#define LCD_COMMAND_NVGAMCTRL	0xe1	// Negative Voltage Gamma Control (14byte)
#define LCD_COMMAND_DGMLUTR		0xe2	// Digital Gamma Look-up Table for Red (64byte)
#define LCD_COMMAND_DGMLUTB		0xe3	// Digital Gamma Look-up Table for Blue (64byte)
#define LCD_COMMAND_GATECTRL	0xe4	// Gate control (3byte)
#define   LCD_DATA1_NL(n)			(n)			// Set the number of gate line / 8
#define   LCD_DATA2_SCN(n)			(n)			// set the first scan line / 8
#define   LCD_DATA3_TMG				(1<<4)		// Gate mirror selection
#define   LCD_DATA3_SM				(1<<2)		// Gate interlace mode selection
#define   LCD_DATA3_GS				(1<<0)		// Gate scan direction

#define LCD_COMMAND_SPI2EN		0xe7	// SPI2 enable (1byte)
#define   LCD_DATA_SPI2EN			(1<<4)		// 2 data lane enable control
#define   LCD_DATA_SPIRD			(1<<0)		// SPI read enable for command table 2

#define LCD_COMMAND_PWCTRL2		0xe8	// Power Control 2 (1byte)
#define   LCD_DATA_SBCLK(n)			((n)<<4)	// Source booster clock selection
#define   LCD_DATA_STP14CK(n)		((n)<<0)	// STP14(AVDD/AVCL) booster clock selection

#define LCD_COMMAND_EQCTRL		0xe9	// Equalize Time Control (1byte)
#define   LCD_DATA1_SEQ(n)			(n)			// Source Equalize Time
#define   LCD_DATA2_SPRET(n)		(n)			// Source Pre-drive Time
#define   LCD_DATA3_GEQ(n)			(n)			// Gate Equalize Time

#define LCD_COMMAND_PROMCTRL	0xec	// Program control (1byte)
#define   LCD_DATA_PROMCTRL			0x01		//

#define LCD_COMMAND_PROMEN		0xfa	// Program Mode Enable (4byte)
#define   LCD_SET_PROMEN(b,en)			b[0]=0x5a; \
										b[1]=0x69; \
										b[2]=0xee; \
										b[3]=(en)<<2;

#define LCD_COMMAND_NVMSET		0xfc	// NVM Setting (2byte)
#define   LCD_DATA1_ADD(n)			(n)			// NVM address setting
#define   LCD_DATA2_D(n)			(n)			// Data setting of NVM address

#define LCD_COMMAND_PROMACT		0xfe	// Program Action (2byte)
#define   LCD_SET_PROMACT(b)			b[0]=0x19; \
										b[1]=0xa5;

#endif /* LCD_SPEC_ST7789V2_H_ */
