//
//  lcd_touch.c	Part of Graphic LCD module I/O library, Touch panel interface
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//		Touch sense or mouse, joy stick devices.
//      Coordinate axis matching, Display and Touch screen.
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.15 v0.0  First cording, default process
//

//#define DEBUG_AXIS

#include "lcd_draw.h"
#include "lcd_touch_driver.h"
#ifdef DEBUG_AXIS
#include <stdio.h>
#endif

//
// Macro
//
#define MAX(a,b)	((a) > (b) ? (a) : (b))
#define MIN(a,b)	((a) < (b) ? (a) : (b))

//
// Local control block
//
static int	openFlag;

//
// Open
//
__attribute__((weak))					// overridable
int lcd_touchOpen(void) {
	int	r;

	r = 1;
	if (!openFlag) {
			r = lcd_touchDriverStart();
			if (r == 0) {
				openFlag = 1;
			}
	}
	return(r);
}

//
// Close
//
__attribute__((weak))					// overridable
void lcd_touchClose(void) {
	int	r;

	if (openFlag) {
		r = lcd_touchDriverStop();
		if (r == 0) {
			openFlag = 0;
		}
	}
	return;
}

//
// Get touch point
//
__attribute__((weak))					// overridable
short lcd_touchPoint(short *px, short *py) {
	static int	x;
	static int	y;
	static int	z;
	int		f;
	short	p[3];
	int		r;

	// Hide open
	if (!openFlag) {
		lcd_touchOpen();
	}

	// Get touch point
	r = lcd_touchDriverGet(p);
	if (r < 0) {						// Error
		f = -1;

	} else if (r == 0) {				// Ok
#ifdef DEBUG_AXIS
		{
			char buf[32];

			sprintf(buf, "%6d,%6d,%6d ", p[0], p[1], p[2]);
			lcd_drawStr(0,0, LCD_COLOR_WHITE, LCD_COLOR_BLACK, buf);
		}
#endif
		// Module/device coordinate convert
		x = p[0];
		y = p[1];
		z = p[2];
#ifdef AA_Y1
		y = (long)LCD_TOUCH_DRIVER_POINT_MAX * (y - AA_Y1) / (AA_Y2 - AA_Y1);
		x = (long)LCD_TOUCH_DRIVER_POINT_MAX * (x - AA_X1) / (AA_X2 - AA_X1);
		z = (long)LCD_TOUCH_DRIVER_POINT_MAX * (z - AA_Z1) / (AA_Z2 - AA_Z1);
#endif
#ifdef AA_ZY
		z = z + (y * AA_ZY / 100) + (x * AA_ZX / 100);
#endif


		// Screen coordinate convert
#ifdef LCD_LANDSCAPE_MODE
		x =                      (long)x * LCD_SCREEN_WIDTH  / LCD_TOUCH_DRIVER_POINT_MAX;
		y = LCD_SCREEN_HEIGHT - ((long)y * LCD_SCREEN_HEIGHT / LCD_TOUCH_DRIVER_POINT_MAX) - 1;
#else
		x = (long)x * LCD_SCREEN_WIDTH  / LCD_TOUCH_DRIVER_POINT_MAX;
		y = (long)y * LCD_SCREEN_HEIGHT / LCD_TOUCH_DRIVER_POINT_MAX;
#endif
		z = (long)z * 100               / LCD_TOUCH_DRIVER_POINT_MAX;

#ifdef LCD_UPSIDE_DOWN_MODE
		x = LCD_SCREEN_WIDTH  - x;
		y = LCD_SCREEN_HEIGHT - y;
#endif

		// Limit check
		x = MAX(x, 0);
		x = MIN(x, LCD_SCREEN_WIDTH-1);
		y = MAX(y, 0);
		y = MIN(y, LCD_SCREEN_HEIGHT-1);
		f = z;

	} else {							// Busy
		f = z;							// Before value return
	}

	// Set values
	if (px != NULL) {
		*px = x;
	}
	if (py != NULL) {
		*py = y;
	}

	return(f);
}
