//
//  lcd_touch_driver_4wire.c	Part of Graphic LCD module I/O library, 4-Wire touch screen driver
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		I/O control for Graphic LCD module.
//		4 wire touch screen control.
//      Dynamic curcuit configurate and measure.
//      Non block mode.
//
//      Support LCD Module (LCM):
//          General
//
//  Refernce:
//      none
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.21 v0.0  First cording
//

#include "lcd_touch_driver.h"			// API
#include "lcd_touch_port.h"				// Port access

#ifdef LCD_DRIVER_4WIRE_TOUCH

//
// Config
//
#define ADC_RESOLUTION		4096		// 12bit ADC config (Do not change)
#define ADC_NOCONTACT_RATE	2			// [%] Push contact threshold, absolute value on device
#define SCAN_PERIOD_MIN		10			// [ms]
#define ADC_NOCONTACT		((int)(ADC_RESOLUTION * ADC_NOCONTACT_RATE / 100))

//
// Macro
//
#define MAX(a,b)	((a) > (b) ? (a) : (b))
#define MIN(a,b)	((a) < (b) ? (a) : (b))

//
// Start
//
int	lcd_touchDriverStart(void){
	lcd_touchPortStart();
	return(0);
}

//
// Get
//
int	lcd_touchDriverGet(short point[3]){
	static enum {
		SCAN_STATE_IDLE = 0,
		SCAN_STATE_Z_NEG,
		SCAN_STATE_Z_POS,
		SCAN_STATE_Y,
		SCAN_STATE_X,
		SCAN_STATE_END,
		SCAN_STATE_WAIT_PERIOD,
		SCAN_STATE_WAIT_PERIOD2,
	}	scanState;
	static short	x;
	static short	y;
	static short	z;
	static short	z1;
	static short	z2;
	static uint32_t	scanPeriod;
	int				r;

	//
	// State machine
	//
	r = 1;								// Busy
	switch(scanState){
	case SCAN_STATE_IDLE:
		// Change circuit Z sense
		lcd_touchPortSet(PORT_YU, PORT_OUTPUT, 1);
		lcd_touchPortSet(PORT_XR, PORT_OUTPUT, 0);
		lcd_touchPortSet(PORT_YD, PORT_ANALOG, 0);
		lcd_touchPortSet(PORT_XL, PORT_ANALOG, 0);
		scanPeriod = lcd_touchPortGetTick() + SCAN_PERIOD_MIN;
		scanState  = SCAN_STATE_Z_NEG;
		break;

	case SCAN_STATE_Z_NEG:
		z1 = lcd_touchPortGetAnalog(PORT_YD);
		if (z1 > 0) {
			scanState = SCAN_STATE_Z_POS;
		}
		break;

	case SCAN_STATE_Z_POS:
		z2 = lcd_touchPortGetAnalog(PORT_XL);
		if (z2 > 0) {

			z = ADC_RESOLUTION - (z1-z2) - 1;	// Reverse
			if (z <= ADC_NOCONTACT) {			// Not touch
				z = 0;
//				point[0] = x;
//				point[1] = y;
				point[2] = 0;
				r = 0;							// point is valid
				scanState = SCAN_STATE_WAIT_PERIOD;
			} else {
				// Change circuit Y sense
				lcd_touchPortSet(PORT_YU, PORT_OUTPUT, 1);
				lcd_touchPortSet(PORT_XR, PORT_INPUT,  0);
				lcd_touchPortSet(PORT_YD, PORT_OUTPUT, 0);
				lcd_touchPortSet(PORT_XL, PORT_ANALOG, 0);
				scanState = SCAN_STATE_Y;
			}
		}
		break;

	case SCAN_STATE_Y:
		y = lcd_touchPortGetAnalog(PORT_XL);
		if (y > 0) {
			y = ADC_RESOLUTION - y;				// Reverse

			// Change circuit X sense
			lcd_touchPortSet(PORT_YU, PORT_INPUT,  0);
			lcd_touchPortSet(PORT_XR, PORT_OUTPUT, 1);
			lcd_touchPortSet(PORT_YD, PORT_ANALOG, 0);
			lcd_touchPortSet(PORT_XL, PORT_OUTPUT, 0);
			scanState = SCAN_STATE_X;
		}
		break;

	case SCAN_STATE_X:
		x = lcd_touchPortGetAnalog(PORT_YD);
		if (x > 0) {
			// Change circuit Idle
			lcd_touchPortSet(PORT_YU, PORT_INPUT, 0);
			lcd_touchPortSet(PORT_XR, PORT_INPUT, 0);
			lcd_touchPortSet(PORT_YD, PORT_INPUT, 0);
			lcd_touchPortSet(PORT_XL, PORT_INPUT, 0);
			scanState = SCAN_STATE_END;
		}
		break;

	case SCAN_STATE_END:
		scanState = SCAN_STATE_WAIT_PERIOD;
		break;

	case SCAN_STATE_WAIT_PERIOD:
		if (scanPeriod < lcd_touchPortGetTick()) {
			scanState = SCAN_STATE_IDLE;
		}
		break;

	case SCAN_STATE_WAIT_PERIOD2:
		if (scanPeriod < lcd_touchPortGetTick()) {
			scanPeriod = lcd_touchPortGetTick() + SCAN_PERIOD_MIN;
			scanState  = SCAN_STATE_Z_NEG;
		}
		break;

	default:
		break;
	}

	//
	// Return values
	//
	if (scanState == SCAN_STATE_END) {

#ifdef AA_Y1
		y = (long)ADC_RESOLUTION * (y - AA_Y1) / (AA_Y2 - AA_Y1);
		x = (long)ADC_RESOLUTION * (x - AA_X1) / (AA_X2 - AA_X1);
		z = (long)ADC_RESOLUTION * (z - AA_Z1) / (AA_Z2 - AA_Z1);
#endif
#ifdef AA_ZY
		z = z + (y * AA_ZY / 100) + (x * AA_ZX / 100);
#endif
		x = MAX(x, 0);
		x = MIN(x, ADC_RESOLUTION-1);
		y = MAX(y, 0);
		y = MIN(y, ADC_RESOLUTION-1);
		z = MAX(z, 0);
		z = MIN(z, ADC_RESOLUTION-1);
		point[0] = x;
		point[1] = y;
		point[2] = z;
		r = 0;			// point is valid
	}

	return(r);
}

//
// Stop
//
int	lcd_touchDriverStop(void){
	lcd_touchPortStop();
	return(0);
}

#endif	// LCD_DRIVER_4WIRE_TOUCH

