//
//  lcd_touch_driver_XPT2046.c	Part of Graphic LCD module I/O library, Touch Screen Controller XPT2046
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//   Reference:
//      SHENZHEN XPTEK TECHNOLOGY CO.,LTD
//      XPT2046 Touch Screen Controller
//          Datasheet
//              Version     : none
//              Document No.: none
//              Date        : 2007.05
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.03.13 v0.0  First cording
//

#include "lcd_touch_driver.h"			// API
#include "lcd_touch_port.h"				// Port access
#include "lcd_spec_XPT2046.h"			// Controller commands

#ifdef LCD_DRIVER_XPT2046

//
// Config
//
#define ADC_RESOLUTION		4096		// 12bit ADC config (Do not change)
#define ADC_NOCONTACT_RATE	1			// [%] Push contact threshold, absolute value on device
#define SCAN_PERIOD_MIN		10			// [ms]
#define ADC_NOCONTACT		((int)(ADC_RESOLUTION * ADC_NOCONTACT_RATE / 100))

//
// Macro
//
#define MAX(a,b)	((a) > (b) ? (a) : (b))
#define MIN(a,b)	((a) < (b) ? (a) : (b))

//
// Internal
//
static int getAxis(int ch) {
	unsigned char	a[1];
	unsigned char	b[2];
	int				r;

	lcd_touchPortCS(0);
	a[0] = LCD_CB_START
		 | ch
		 | LCD_CB_MODE_12bit
		 | LCD_CB_DFR
		 | LCD_CB_PD_AUTO
		 ;
	if (lcd_touchPortSend(sizeof(a), a) == sizeof(a)) {
		if (lcd_touchPortRecv(sizeof(b), b) == sizeof(b)) {
			r = (b[0] << 8) | b[1];
			r = (r >> 3) & 0x0fff;
		} else {
			r = -1;
		}
	} else {
		r = -2;
	}
	lcd_touchPortCS(1);

	return(r);
}

//
// API
//
int lcd_touchDriverStart(void){
	return(0);
};

int lcd_touchDriverStop(void){
	return(0);
};

int lcd_touchDriverGet(short point[3]){
	static enum {
		SCAN_STATE_IDLE = 0,
		SCAN_STATE_Z_NEG,
		SCAN_STATE_Z_POS,
		SCAN_STATE_Y,
		SCAN_STATE_X,
		SCAN_STATE_END,
		SCAN_STATE_WAIT_PERIOD,
	}	scanState;
	static short	x;
	static short	y;
	static short	z;
	static short	z1;
	static short	z2;
	static uint32_t	scanPeriod;
	int				r;

	//
	// State machine
	//
	r = 1;								// Busy
	switch(scanState){
	case SCAN_STATE_IDLE:
		// Change circuit Z sense
		scanPeriod = lcd_touchPortGetTick() + SCAN_PERIOD_MIN;
		scanState  = SCAN_STATE_Z_NEG;
		break;

	case SCAN_STATE_Z_NEG:
		z1 = getAxis(LCD_CB_A_Z1);
		if (z1 > 0) {
			if (z1 <= ADC_NOCONTACT) {			// Not touch
				z = 0;
//				point[0] = x;
//				point[1] = y;
				point[2] = 0;
				r = 0;							// point is valid
				scanState = SCAN_STATE_WAIT_PERIOD;
			} else {
				scanState = SCAN_STATE_Z_POS;
			}
		}
		break;

	case SCAN_STATE_Z_POS:
		z2 = getAxis(LCD_CB_A_Z2);
		if (z2 > 0) {
			z = ADC_RESOLUTION - (z2 - z1) - 1;	// Reverse
			scanState = SCAN_STATE_Y;
		}
		break;

	case SCAN_STATE_Y:
		y = getAxis(LCD_CB_A_Y);
		if (y > 0) {
			y = ADC_RESOLUTION - y;				// Reverse
			scanState = SCAN_STATE_X;
		}
		break;

	case SCAN_STATE_X:
		x = getAxis(LCD_CB_A_X);
		if (x > 0) {
			scanState = SCAN_STATE_END;
		}
		break;

	case SCAN_STATE_END:
		scanState = SCAN_STATE_WAIT_PERIOD;
		break;

	case SCAN_STATE_WAIT_PERIOD:
		if (scanPeriod < lcd_touchPortGetTick()) {
			scanState = SCAN_STATE_IDLE;
		}
		break;

	default:
		break;
	}

	//
	// Return values
	//
	if (scanState == SCAN_STATE_END) {
		x = MAX(x, 0);
		x = MIN(x, ADC_RESOLUTION-1);
		y = MAX(y, 0);
		y = MIN(y, ADC_RESOLUTION-1);
		z = MAX(z, 0);
		z = MIN(z, ADC_RESOLUTION-1);
		point[0] = x;
		point[1] = y;
		point[2] = z;
		r = 0;			// point is valid
	}

	return(r);
}

#endif
