//
//  lcd_touch_port_STM32CubeF4.c	Part of Graphic LCD module I/O library, Touch screen port I/O
//  Copyright(C)2022 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Host device   : STM32F4 series
//    Library       : STM32Cube F4 HAL
//    IDE tool      : STM32CubeIDE & STM32CubeMX
//
//	Settings
//    STM32CubeMX:
//      Pinout & Configuration -> Analog -> ADC* -> Parameter Settings
//          All default value
//
//      Pinout & Configuration -> System Core -> GPIO
//
//      Pin Name  output level     GPIO mode        Pull-up/down       Maximum output speed  User Label
//      --------  ------------  ----------------  -------------------  --------------------  ----------
//      P**             *       Output Push Pull  No pull-up and down           *            LCM_YU
//      P**             *       Output Push Pull  No pull-up and down           *            LCM_XR
//      P**            N/A      Analog mode       No pull-up and down          N/A           LCM_YD
//      P**            N/A      Analog mode       No pull-up and down          N/A           LCM_XL
//      --------  ------------  ----------------  -------------------  --------------------  ----------
//      P**            High     Output Push Pull  No pull-up and down          High          LCM_TP_SCK
//      P**             *       Input             No pull-up and down          N/A           LCM_TP_SDI
//      P**            Low      Output Push Pull  No pull-up and down          High          LCM_TP_SDO
//      P**            High     Output Push Pull  No pull-up and down          High          LCM_TP_CS
//
//    STM32CubeIDE:
//      Project properties -> C/C++ Build -> setting -> MCU GCC Compiler -> Preprocessor -> Define symbols
//        Add define symbol
//             "LCD_TOUCH_ADC_CB=hadc*"
//             "LCD_TOUCH_SPI_CB=hspi*"		<- option, not defineed is software SPI
//                 Value is ADC/SPI control block name in main.c(STM32CubeMX generated file)
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2022.02.21 v0.0  First cording
//

#include <string.h>
#include "lcd_touch_port.h"


//---------------------------------------------------------------------
//
//						STM32Cube HAL Drivers
//
//---------------------------------------------------------------------
#ifdef USE_HAL_DRIVER

#include "main.h"
//
// HAL speed up cheat
//
#define HAL_GPIO_SetPin(port, pin)			port->BSRR = pin
#define HAL_GPIO_ResetPin(port, pin)		port->BSRR = (uint32_t)pin << 16U
#define HAL_GPIO_Write(port, pins, data)	port->ODR = (port->ODR & ~pins) | (data & pins)
#define HAL_GPIO_Read(port, pins)			(port->IDR & pins)

//
//	Open
//
void lcd_touchPortStart(void) {
	// use STM32CubeMX initializer
	return;
}

//
// Stop
//
void lcd_touchPortStop(void) {
	return;
}

//
// Now time [ms]
//
uint32_t lcd_touchPortGetTick(void) {
	return(HAL_GetTick());
}


#ifdef LCM_YD_Pin
//-----------------------------------------------------------------------------
// Analog interface
//-----------------------------------------------------------------------------

//
// Config
//
//#define ADC_SAMPLE_TIME	ADC_SAMPLETIME_3CYCLES
#define ADC_SAMPLE_TIME		ADC_SAMPLETIME_480CYCLES
#define ADC_TIMEOUT			1			// [ms] ADC convert time limit

//
// HAL control blocks
//
#ifndef LCD_TOUCH_ADC_CB				// Control block
#define LCD_TOUCH_ADC_CB	hadc2		// default is ADC2
#endif
extern ADC_HandleTypeDef LCD_TOUCH_ADC_CB;

//
// Local subroutines
//
static void port2gpioPin(enum lcd_touchPortId port, GPIO_TypeDef **gpio, uint32_t *pin) {
	switch(port) {
	case PORT_YD:
		*gpio = LCM_YD_GPIO_Port;		// define from STM32CubeMX
		*pin  = LCM_YD_Pin;
		break;
	case PORT_YU:
		*gpio = LCM_YU_GPIO_Port;
		*pin  = LCM_YU_Pin;
		break;
	case PORT_XL:
		*gpio = LCM_XL_GPIO_Port;
		*pin  = LCM_XL_Pin;
		break;
	case PORT_XR:
		*gpio = LCM_XR_GPIO_Port;
		*pin  = LCM_XR_Pin;
		break;
	default:
		*gpio = NULL;
		*pin  = 0;
		break;
	}
	return;
}

static uint32_t port2analogChannel(enum lcd_touchPortId port){
	uint32_t		c;
	GPIO_TypeDef	*g;
	uint32_t		p;

	c = 0;
	port2gpioPin(port, &g, &p);
	if (g == GPIOA) {
		if        (p == GPIO_PIN_0) {
			c = ADC_CHANNEL_0;
		} else if (p == GPIO_PIN_1) {
			c = ADC_CHANNEL_1;
		} else if (p == GPIO_PIN_2) {
			c = ADC_CHANNEL_2;
		} else if (p == GPIO_PIN_3) {
			c = ADC_CHANNEL_3;
		} else if (p == GPIO_PIN_4) {
			c = ADC_CHANNEL_4;
		} else if (p == GPIO_PIN_5) {
			c = ADC_CHANNEL_5;
		} else if (p == GPIO_PIN_6) {
			c = ADC_CHANNEL_6;
		} else if (p == GPIO_PIN_7) {
			c = ADC_CHANNEL_7;
		}
	} else if (g == GPIOB) {
		if        (p == GPIO_PIN_0) {
			c = ADC_CHANNEL_8;
		} else if (p == GPIO_PIN_1) {
			c = ADC_CHANNEL_9;
		}
	} else if (g == GPIOC) {
		if        (p == GPIO_PIN_0) {
			c = ADC_CHANNEL_10;
		} else if (p == GPIO_PIN_1) {
			c = ADC_CHANNEL_11;
		} else if (p == GPIO_PIN_2) {
			c = ADC_CHANNEL_12;
		} else if (p == GPIO_PIN_3) {
			c = ADC_CHANNEL_13;
		} else if (p == GPIO_PIN_4) {
			c = ADC_CHANNEL_14;
		} else if (p == GPIO_PIN_5) {
			c = ADC_CHANNEL_15;
		}
	} else if (g == GPIOF) {
		if        (p == GPIO_PIN_3) {
			c = ADC_CHANNEL_9;
		} else if (p == GPIO_PIN_4) {
			c = ADC_CHANNEL_14;
		} else if (p == GPIO_PIN_5) {
			c = ADC_CHANNEL_15;
		} else if (p == GPIO_PIN_6) {
			c = ADC_CHANNEL_4;
		} else if (p == GPIO_PIN_7) {
			c = ADC_CHANNEL_5;
		} else if (p == GPIO_PIN_8) {
			c = ADC_CHANNEL_6;
		} else if (p == GPIO_PIN_9) {
			c = ADC_CHANNEL_7;
		} else if (p == GPIO_PIN_10) {
			c = ADC_CHANNEL_8;
		}
	}
	return(c);
}

//
// Setting
//
void lcd_touchPortSet(enum lcd_touchPortId port, enum lcd_touchPortMode mode, int onOff){
	GPIO_TypeDef		*g;
	GPIO_InitTypeDef	p = {0};

	port2gpioPin(port, &g, &(p.Pin));
	p.Pull      = GPIO_NOPULL;
	p.Speed     = GPIO_SPEED_FREQ_MEDIUM;
	switch(mode) {
	case PORT_INPUT:
		p.Mode = GPIO_MODE_INPUT;
		break;
	case PORT_OUTPUT:
		p.Mode = GPIO_MODE_OUTPUT_PP;
		break;
	case PORT_ANALOG:
		p.Mode = GPIO_MODE_ANALOG;
		break;
	default:
		break;
	}
	HAL_GPIO_Init(g, &p);

	if (mode == PORT_OUTPUT) {
		if (onOff) {
			HAL_GPIO_WritePin(g, p.Pin, GPIO_PIN_SET);
		} else {
			HAL_GPIO_WritePin(g, p.Pin, GPIO_PIN_RESET);
		}
	}

	return;
}

//
// Analog read
//

int lcd_touchPortGetAnalog(enum lcd_touchPortId port){
	int					r;
	int					v;
	HAL_StatusTypeDef	s;
	ADC_ChannelConfTypeDef	conf = {0};

	r = 0;		// Busy
	conf.Channel      = port2analogChannel(port);
	conf.Rank         = 1;
	conf.SamplingTime = ADC_SAMPLE_TIME;

	if (HAL_ADC_ConfigChannel(&LCD_TOUCH_ADC_CB, &conf) != HAL_OK) {
		r = -1;
	} else {
		HAL_ADC_Start(&LCD_TOUCH_ADC_CB);
		s = HAL_ADC_PollForConversion(&LCD_TOUCH_ADC_CB, ADC_TIMEOUT);
		v = HAL_ADC_GetValue(&LCD_TOUCH_ADC_CB);
		HAL_ADC_Stop(&LCD_TOUCH_ADC_CB);
		if (s == HAL_OK) {
			r = v;
		}
	}
	return(r);
}


#elif defined(LCM_TP_SCK_Pin)
//-----------------------------------------------------------------------------
// Digital interface
//-----------------------------------------------------------------------------

#ifdef LCD_TOUCH_SPI_CB				// Control block name "hspiX" in main.h
//
// Hardware mode
//
extern SPI_HandleTypeDef LCD_TOUCH_SPI_CB;

size_t lcd_touchPortSend(size_t len, unsigned char *data) {
	HAL_StatusTypeDef s;

	s = HAL_SPI_Transmit(&LCD_TOUCH_SPI_CB, (uint8_t *)data, len, 1);
	if (s != HAL_OK) {
//		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);	// Error LED
		len = 0;
	}
	return(len);
}

size_t lcd_touchPortRecv(size_t len, unsigned char *data) {
	HAL_StatusTypeDef s;

	memset(data, 0, len);
	s = HAL_SPI_TransmitReceive(&LCD_TOUCH_SPI_CB, (uint8_t *)data, (uint8_t *)data, len, 2);
	if (s != HAL_OK) {
//		HAL_GPIO_WritePin(LD2_GPIO_Port, LD2_Pin, GPIO_PIN_SET);	// Error LED
		len = 0;
	}
	return(len);
}

#else
//
// Software mode
//
size_t lcd_touchPortSend(size_t len, unsigned char *data) {
	int	i;
	int	j;

	for(j = 0; j < len; j++) {
		int	d;

		d = *data++;
		for(i = 0; i < 8; i++) {
			HAL_GPIO_ResetPin(LCM_TP_SCK_GPIO_Port, LCM_TP_SCK_Pin);
			if (d & 0x80) {
				HAL_GPIO_SetPin(LCM_TP_SDO_GPIO_Port, LCM_TP_SDO_Pin);
			} else {
				HAL_GPIO_ResetPin(LCM_TP_SDO_GPIO_Port, LCM_TP_SDO_Pin);
			}
			HAL_GPIO_SetPin(LCM_TP_SCK_GPIO_Port, LCM_TP_SCK_Pin);
			d = d << 1;
		}
	}
	return(len);
}

size_t lcd_touchPortRecv(size_t len, unsigned char *data) {
	int	i;
	int	j;

	for(j = 0; j < len; j++) {
		int	d;

		d = 0;
		for(i = 0; i < 8; i++) {
			HAL_GPIO_ResetPin(LCM_TP_SCK_GPIO_Port, LCM_TP_SCK_Pin);
			d = d << 1;
			if (HAL_GPIO_Read(LCM_TP_SDI_GPIO_Port, LCM_TP_SDI_Pin)) {
				d = d | 1;
			}
			HAL_GPIO_SetPin(LCM_TP_SCK_GPIO_Port, LCM_TP_SCK_Pin);
		}
		*data++ = d;
	}
	return(len);
}

#endif

//
// Port not define -> dummy routine
//
#else
void	lcd_touchPortSet(enum lcd_touchPortId port, enum lcd_touchPortMode mode, int onOff) {return;}
int		lcd_touchPortGetAnalog(enum lcd_touchPortId port) { return(0); }
size_t	lcd_touchPortSend(size_t len, unsigned char *data) { return(0); }
size_t	lcd_touchPortRecv(size_t len, unsigned char *data) { return(0); }

#endif

#ifdef LCM_TP_CS_Pin
void lcd_touchPortCS(int flag) {
	if (flag) {
		HAL_GPIO_SetPin(LCM_TP_CS_GPIO_Port, LCM_TP_CS_Pin);
	} else {
		HAL_GPIO_ResetPin(LCM_TP_CS_GPIO_Port, LCM_TP_CS_Pin);
	}
	return;
}
#else
void lcd_touchPortCS(int flag) {	// Dummy
	return;
}
#endif

//---------------------------------------------------------------------
//
//						StdPeriph_Driver
//
//---------------------------------------------------------------------
#else	// USE_HAL_DRIVER

#error "Not supported HAL"

#endif	// USE_HAL_DRIVER
